from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get, deep_walk

gcp_access_attempts_violating_vpc_service_controls_tests: list[RuleTest] = [
    RuleTest(
        name="Other Event",
        expected_result=False,
        log={
            "insertId": "12345",
            "logName": "projects/test-project/logs/cloudaudit.googleapis.com%2Factivity",
            "operation": {
                "first": True,
                "id": "operation-abcdefg-1234567",
                "last": True,
                "producer": "compute.googleapis.com",
            },
            "p_any_ip_addresses": ["1.2.3.4"],
            "p_event_time": "2023-03-08 18:52:52.114",
            "p_log_type": "GCP.AuditLog",
            "p_parse_time": "2023-03-08 18:54:14.595",
            "p_row_id": "5e7586fcbb73fdeed985ebeb16bd0c",
            "p_source_id": "4fc88a5a-2d51-4279-9c4a-08fa7cc52566",
            "p_source_label": "gcplogsource",
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {"principalEmail": "user1@company.io"},
                "authorizationInfo": [
                    {
                        "granted": True,
                        "permission": "compute.subnetworks.update",
                        "resourceAttributes": {
                            "name": "projects/test-project/regions/us-central1/subnetworks/default",
                            "service": "compute",
                            "type": "compute.subnetworks",
                        },
                    },
                ],
                "methodName": "v1.compute.subnetworks.patch",
                "request": {
                    "@type": "type.googleapis.com/compute.subnetworks.patch",
                    "fingerprint": "/�/��\x03��",
                    "logConfig": {"enable": True},
                },
                "requestMetadata": {
                    "callerIP": "1.2.3.4",
                    "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/110.0.0.0 Safari/537.36,gzip(gfe),gzip(gfe)",
                    "destinationAttributes": {},
                    "requestAttributes": {
                        "auth": {},
                        "reason": "8uSywAYQGg5Db2xpc2V1bSBGbG93cw",
                        "time": "2023-03-08T18:52:52.558899Z",
                    },
                },
                "resourceName": "projects/test-project/regions/us-central1/subnetworks/default",
                "response": {
                    "@type": "type.googleapis.com/operation",
                    "endTime": "2023-03-08T10:52:52.510-08:00",
                    "id": "9876543210",
                    "insertTime": "2023-03-08T10:52:52.509-08:00",
                    "name": "operation-abcdefg-1234567",
                    "operationType": "compute.subnetworks.patch",
                    "progress": "100",
                    "region": "https://www.googleapis.com/compute/v1/projects/test-project/regions/us-central1",
                    "selfLink": "https://www.googleapis.com/compute/v1/projects/test-project/regions/us-central1/operations/operation-abcdefg-1234567",
                    "selfLinkWithId": "https://www.googleapis.com/compute/v1/projects/test-project/regions/us-central1/operations/9876543210",
                    "startTime": "2023-03-08T10:52:52.510-08:00",
                    "status": "DONE",
                    "targetId": "567854321",
                    "targetLink": "https://www.googleapis.com/compute/v1/projects/test-project/regions/us-central1/subnetworks/default",
                    "user": "user1@company.io",
                },
                "serviceName": "compute.googleapis.com",
            },
            "receiveTimestamp": "2023-03-08 18:52:52.72",
            "resource": {
                "labels": {
                    "location": "us-central1",
                    "project_id": "test-project",
                    "subnetwork_id": "567854321",
                    "subnetwork_name": "default",
                },
                "type": "gce_subnetwork",
            },
            "severity": "NOTICE",
            "timestamp": "2023-03-08 18:52:52.114",
        },
    ),
    RuleTest(
        name="VPC control violation",
        expected_result=True,
        log={
            "insertId": "13ogcded7jh2",
            "insertid": "15wr7lbb6j",
            "logName": "projects/gcpproject/logs/cloudaudit.googleapis.com%2Fpolicy",
            "logname": "projects/gcpproject/logs/cloudaudit.googleapis.com%2Factivity",
            "p_any_ip_addresses": ["1.2.3.4"],
            "p_event_time": "2023-03-09 10:53:14.929",
            "p_log_type": "GCP.AuditLog",
            "p_parse_time": "2023-03-09 10:54:14.363",
            "p_row_id": "7ad218d42253b7e6f78cc0ed16be37",
            "p_source_id": "4fc88a5a-2d51-4279-9c4a-08fa7cc52566",
            "p_source_label": "gcplogsource",
            "protoPayload": {
                "at_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {"principalEmail": "user1@serviceaccount.gcp.com"},
                "metadata": {
                    "at_type": "type.googleapis.com/google.cloud.audit.VpcServiceControlAuditMetadata",
                    "deviceState": "Unknown",
                    "ingressViolations": [
                        {
                            "servicePerimeter": "accessPolicies/123456789012/servicePerimeters/test_perimeter",
                            "targetResource": "projects/197946410614",
                            "targetResourcePermissions": ["NO_PERMISSIONS"],
                        },
                    ],
                    "resourceNames": ["projects/_/buckets/test-restricted-bucket/objects/test1.txt"],
                    "securityPolicyInfo": {
                        "organizationId": "645568414902",
                        "servicePerimeterName": "accessPolicies/123456789012/servicePerimeters/test_perimeter",
                    },
                    "violationReason": "NO_MATCHING_ACCESS_LEVEL",
                    "vpcServiceControlsUniqueId": "gBc-wuGVCapNMnTUePoHos_VyJmr3CsMKlr48kVa4b6XpsT_OWKRng",
                },
                "methodName": "google.storage.objects.get",
                "requestMetadata": {"callerIp": "1.2.3.4", "destinationAttributes": {}, "requestAttributes": {}},
                "resourceName": "projects/197946410614",
                "serviceName": "storage.googleapis.com",
                "status": {
                    "code": 7,
                    "details": [
                        {
                            "at_type": "type.googleapis.com/google.rpc.PreconditionFailure",
                            "violations": [
                                {
                                    "description": "gBc-wuGVCapNMnTUePoHos_VyJmr3CsMKlr48kVa4b6XpsT_OWKRng",
                                    "type": "VPC_SERVICE_CONTROLS",
                                },
                                {
                                    "description": "gCc-wuJa334DJ9940ssdiw_V8400skgjj3912500sldgjzh_LGJANr",
                                    "type": "OTHER_CONTROL_VIOLATION",
                                },
                            ],
                        },
                    ],
                    "message": "Request is prohibited by organization's policy. vpcServiceControlsUniqueIdentifier: gBc-wuGVCapNMnTUePoHos_VyJmr3CsMKlr48kVa4b6XpsT_OWKRng",
                },
            },
            "receiveTimestamp": "2023-03-09T16:28:42.567340480Z",
            "resource": {
                "labels": {
                    "method": "google.storage.objects.get",
                    "project_id": "gcpproject",
                    "service": "storage.googleapis.com",
                },
                "type": "audited_resource",
            },
            "severity": "ERROR",
            "timestamp": "2023-03-09T16:28:40.890430163Z",
        },
    ),
]


@panther_managed
class GCPAccessAttemptsViolatingVPCServiceControls(Rule):
    default_description = "An access attempt violating VPC service controls (such as Perimeter controls) has been made."
    display_name = "GCP Access Attempts Violating VPC Service Controls"
    default_reference = "https://cloud.google.com/vpc-service-controls/docs/troubleshooting#debugging"
    default_severity = Severity.MEDIUM
    log_types = [LogType.GCP_AUDIT_LOG]
    id = "GCP.Access.Attempts.Violating.VPC.Service.Controls-prototype"
    tests = gcp_access_attempts_violating_vpc_service_controls_tests

    def rule(self, event):
        severity = deep_get(event, "severity", default="")
        status_code = deep_get(event, "protoPayload", "status", "code", default="")
        violation_types = deep_walk(event, "protoPayload", "status", "details", "violations", "type", default=[])
        if all([severity == "ERROR", status_code == 7, "VPC_SERVICE_CONTROLS" in violation_types]):
            return True
        return False

    def title(self, event):
        actor = deep_get(event, "protoPayload", "authenticationInfo", "principalEmail", default="<ACTOR_NOT_FOUND>")
        method = deep_get(event, "protoPayload", "methodName", default="<METHOD_NOT_FOUND>")
        return f"GCP: [{actor}] performed a [{method}] request that violates VPC Service Controls"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

one_login_unauthorized_access_tests: list[RuleTest] = [
    RuleTest(
        name="Normal Event",
        expected_result=False,
        log={"event_type_id": "8", "user_id": 123456, "user_name": "Bob Cat", "app_name": "confluence"},
    ),
    RuleTest(
        name="User Unauthorized Access Event",
        expected_result=True,
        log={"event_type_id": "90", "user_id": 123456, "user_name": "Bob Cat", "app_name": "confluence"},
    ),
]


@panther_managed
class OneLoginUnauthorizedAccess(Rule):
    id = "OneLogin.UnauthorizedAccess-prototype"
    display_name = "OneLogin Unauthorized Access"
    log_types = [LogType.ONELOGIN_EVENTS]
    tags = ["OneLogin", "Lateral Movement:Use Alternate Authentication Material"]
    reports = {"MITRE ATT&CK": ["TA0008:T1550"]}
    default_severity = Severity.MEDIUM
    default_description = "A OneLogin user was denied access to an app more times than the configured threshold."
    threshold = 10
    dedup_period_minutes = 10
    default_reference = "https://onelogin.service-now.com/kb_view_customer.do?sysparm_article=KB0010420"
    default_runbook = "Analyze the user activity and actions."
    summary_attributes = ["account_id", "user_name", "user_id", "app_name"]
    tests = one_login_unauthorized_access_tests

    def rule(self, event):
        # filter events; event type 90 is an unauthorized application access event id
        return str(event.get("event_type_id")) == "90"

    def title(self, event):
        return f"User [{event.get('user_name', '<UNKNOWN_USER>')}] has exceeded the unauthorized application access attempt threshold"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.helpers.panther_tailscale_helpers import is_tailscale_admin_console_event, tailscale_alert_context

tailscale_https_disabled_tests: list[RuleTest] = [
    RuleTest(
        name="HTTPS Disabled",
        expected_result=True,
        log={
            "event": {
                "action": "DISABLE",
                "actor": {
                    "displayName": "Homer Simpson",
                    "id": "uodc9f3CNTRL",
                    "loginName": "homer.simpson@yourcompany.io",
                    "type": "USER",
                },
                "eventGroupID": "1770eb26fb58fbf67fd8fbfcc2056e66",
                "origin": "ADMIN_CONSOLE",
                "target": {"id": "yoururl.com", "name": "yoururl.com", "property": "HTTPS", "type": "TAILNET"},
            },
            "fields": {"recorded": "2023-07-19 16:10:48.385283827"},
            "p_any_actor_ids": ["uodc9f3CNTRL"],
            "p_any_emails": ["homer.simpson@yourcompany.io"],
            "p_any_usernames": ["andrea.youwakim"],
            "p_event_time": "2023-07-19 16:10:48.172000",
            "p_log_type": "Tailscale.Audit",
            "p_parse_time": "2023-07-19 16:13:56.849092",
            "p_row_id": "5e197fb53834e39eeab7feb9198d04",
            "p_schema_version": 0,
            "p_source_id": "5d65e24a-7ebb-403b-803c-51396e03d201",
            "p_source_label": "Tailscale Audit and Network Logs",
            "p_udm": {},
            "time": "2023-07-19 16:10:48.172000000",
        },
    ),
    RuleTest(
        name="Other Event",
        expected_result=False,
        log={
            "event": {
                "action": "CREATE",
                "actor": {
                    "displayName": "Homer Simpson",
                    "id": "uodc9f3CNTRL",
                    "loginName": "homer.simpson@yourcompany.io",
                    "type": "USER",
                },
                "eventGroupID": "9f880e02981e341447958344b7b4071f",
                "new": {},
                "origin": "ADMIN_CONSOLE",
                "target": {"id": "k6r3fm3CNTRL", "name": "API key", "type": "API_KEY"},
            },
            "fields": {"recorded": "2023-07-19 16:11:41.778839718"},
            "p_any_actor_ids": ["uodc9f3CNTRL"],
            "p_any_emails": ["homer.simpson@yourcompany.io"],
            "p_any_usernames": ["homersimpson"],
            "p_event_time": "2023-07-19 16:11:41.601000",
            "p_log_type": "Tailscale.Audit",
            "p_parse_time": "2023-07-19 16:14:56.865276",
            "p_row_id": "02eaf97ec9caaaabff8882ba19ad1d",
            "p_schema_version": 0,
            "p_source_id": "5d65e24a-7ebb-403b-803c-51396e03d201",
            "p_source_label": "Tailscale Audit and Network Logs",
            "p_udm": {},
            "time": "2023-07-19 16:11:41.601000000",
        },
    ),
]


@panther_managed
class TailscaleHTTPSDisabled(Rule):
    default_description = "A Tailscale User disabled HTTPS settings in your organization's tenant."
    display_name = "Tailscale HTTPS Disabled"
    default_runbook = "Assess if this was done by the user for a valid business reason. Be vigilant to re-enable this setting as it's in the best security interest for your organization's security posture."
    default_reference = "https://tailscale.com/kb/1153/enabling-https/#disable-https"
    default_severity = Severity.HIGH
    log_types = [LogType.TAILSCALE_AUDIT]
    id = "Tailscale.HTTPS.Disabled-prototype"
    tests = tailscale_https_disabled_tests

    def rule(self, event):
        action = deep_get(event, "event", "action", default="<NO_ACTION_FOUND>")
        target_property = deep_get(event, "event", "target", "property", default="<NO_TARGET_PROPERTY_FOUND>")
        return all([action == "DISABLE", target_property == "HTTPS", is_tailscale_admin_console_event(event)])

    def title(self, event):
        user = deep_get(event, "event", "actor", "loginName", default="<NO_USER_FOUND>")
        target_id = deep_get(event, "event", "target", "id", default="<NO_TARGET_ID_FOUND>")
        return f"Tailscale user [{user}] disabled HTTPS for [{target_id}] in your organization’s tenant."

    def alert_context(self, event):
        return tailscale_alert_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

osquery_mac_osx_attacks_tests: list[RuleTest] = [
    RuleTest(
        name="Valid malware discovered",
        expected_result=True,
        log={
            "name": "pack_osx-attacks_Leverage-A_1",
            "action": "added",
            "hostIdentifier": "test-host",
            "columns": {"path": "/Users/johnny/Desktop/Siri.app/Contents/MacOS/Siri", "pid": 100, "name": "Siri"},
        },
    ),
    RuleTest(
        name="Keyboard event taps query is ignored",
        expected_result=False,
        log={
            "name": "pack_osx-attacks_Keyboard_Event_Taps",
            "action": "added",
            "hostIdentifier": "test-host",
            "columns": {
                "path": "/System/Library/CoreServices/Siri.app/Contents/MacOS/Siri",
                "pid": 100,
                "name": "Siri",
            },
        },
    ),
]


@panther_managed
class OsqueryMacOSXAttacks(Rule):
    id = "Osquery.Mac.OSXAttacks-prototype"
    display_name = "macOS Malware Detected with osquery"
    log_types = [LogType.OSQUERY_DIFFERENTIAL]
    tags = ["Osquery", "MacOS", "Malware", "Resource Development:Develop Capabilities"]
    reports = {"MITRE ATT&CK": ["TA0042:T1588"]}
    default_severity = Severity.MEDIUM
    default_description = "Malware has potentially been detected on a macOS system"
    default_runbook = "Check the executable against VirusTotal"
    default_reference = "https://github.com/osquery/osquery/blob/master/packs/osx-attacks.conf"
    summary_attributes = ["name", "hostIdentifier", "action"]
    tests = osquery_mac_osx_attacks_tests

    def rule(self, event):
        if "osx-attacks" not in event.get("name", ""):
            return False
        # There is another rule specifically for this query
        if "Keyboard_Event_Taps" in event.get("name", ""):
            return False
        if event.get("action") != "added":
            return False
        return True

    def title(self, event):
        return f"MacOS malware detected on [{event.get('hostIdentifier')}]"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

asana_workspace_email_domain_added_tests: list[RuleTest] = [
    RuleTest(
        name="new domain",
        expected_result=True,
        log={
            "actor": {
                "actor_type": "user",
                "email": "homer.simpson@example.io",
                "gid": "12345",
                "name": "Homer Simpson",
            },
            "context": {
                "client_ip_address": "12.12.12.12",
                "context_type": "web",
                "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36",
            },
            "created_at": "2022-12-16 19:30:26.15",
            "details": {"new_value": "test.com"},
            "event_category": "admin_settings",
            "event_type": "workspace_associated_email_domain_added",
            "gid": "12345",
            "resource": {"gid": "12345", "name": "Example IO", "resource_type": "workspace"},
        },
    ),
    RuleTest(
        name="other event",
        expected_result=False,
        log={
            "actor": {
                "actor_type": "user",
                "email": "homer.simpson@example.io",
                "gid": "12345",
                "name": "Homer Simpson",
            },
            "context": {
                "client_ip_address": "12.12.12.12",
                "context_type": "web",
                "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36",
            },
            "created_at": "2022-12-16 19:30:26.15",
            "details": {"new_value": "anyone", "old_value": "admins_only"},
            "event_category": "admin_settings",
            "event_type": "workspace_guest_invite_permissions_changed",
            "gid": "12345",
            "resource": {"gid": "12345", "name": "Example IO", "resource_type": "workspace"},
        },
    ),
]


@panther_managed
class AsanaWorkspaceEmailDomainAdded(Rule):
    default_description = "A new email domain has been added to an Asana workspace. Reviewer should validate that the new domain is a part of the organization. "
    display_name = "Asana Workspace Email Domain Added"
    default_reference = (
        "https://help.asana.com/hc/en-us/articles/15901227439515-Email-domain-management-for-Asana-organizations"
    )
    default_severity = Severity.LOW
    log_types = [LogType.ASANA_AUDIT]
    id = "Asana.Workspace.Email.Domain.Added-prototype"
    tests = asana_workspace_email_domain_added_tests

    def rule(self, event):
        return event.get("event_type") == "workspace_associated_email_domain_added"

    def title(self, event):
        workspace = deep_get(event, "resource", "name", default="<WORKSPACE_NOT_FOUND>")
        domain = deep_get(event, "details", "new_value", default="<DOMAIN_NOT_FOUND>")
        actor = deep_get(event, "actor", "email", default="<ACTOR_NOT_FOUND>")
        return f"Asana new email domain [{domain}] added to Workspace [{workspace}] by [{actor}]."



import re

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

zoom_user_promotedto_privileged_role_tests: list[RuleTest] = [
    RuleTest(
        name="Admin Promotion Event",
        expected_result=True,
        log={
            "action": "Batch Update",
            "category_type": "User",
            "operation_detail": "Change Role  - homer.simpson@duff.io: from User to Co-Owner",
            "operator": "admin@duff.io",
            "time": "2022-07-05 20:28:48",
        },
    ),
    RuleTest(
        name="Admin to Admin",
        expected_result=False,
        log={
            "action": "Batch Update",
            "category_type": "User",
            "operation_detail": "Change Role  - homer.simpson@duff.io: from Admin to Co-Owner",
            "operator": "admin@duff.io",
            "time": "2022-07-05 20:28:48",
        },
    ),
    RuleTest(
        name="Admin to Billing Admin",
        expected_result=False,
        log={
            "action": "Batch Update",
            "category_type": "User",
            "operation_detail": "Change Role  - homer.simpson@duff.io: from Admin to Billing Admin",
            "operator": "admin@duff.io",
            "time": "2022-07-05 20:28:48",
        },
    ),
    RuleTest(
        name="Member to Billing Admin Event",
        expected_result=True,
        log={
            "action": "Batch Update",
            "category_type": "User",
            "operation_detail": "Change Role  - homer.simpson@duff.io: from Member to Billing Admin",
            "operator": "admin@duff.io",
            "time": "2022-07-05 20:28:48",
        },
    ),
    RuleTest(
        name="Admin to User",
        expected_result=False,
        log={
            "action": "Batch Update",
            "category_type": "User",
            "operation_detail": "Change Role  - homer.simpson@duff.io: from Co-Owner to User",
            "operator": "admin@duff.io",
            "time": "2022-07-05 20:28:48",
        },
    ),
    RuleTest(
        name="CoOwner to Admin",
        expected_result=False,
        log={
            "action": "Batch Update",
            "category_type": "User",
            "operation_detail": "Change Role  - homer.simpson@duff.io: from Co-Owner to Admin",
            "operator": "admin@duff.io",
            "time": "2022-07-05 20:28:48",
        },
    ),
    RuleTest(
        name="Other Event",
        expected_result=False,
        log={
            "action": "SCIM API - Update",
            "category_type": "User",
            "operation_detail": "Edit User homer.simpson@duff.co  - Change Type: from Basic to Licensed",
            "operator": "admin@duff.co",
            "time": "2022-07-01 22:05:22",
        },
    ),
]


@panther_managed
class ZoomUserPromotedtoPrivilegedRole(Rule):
    default_description = "A Zoom user was promoted to a privileged role."
    display_name = "Zoom User Promoted to Privileged Role"
    default_reference = "https://support.zoom.com/hc/en/article?id=zm_kb&sysparm_article=KB0064983"
    default_severity = Severity.MEDIUM
    log_types = [LogType.ZOOM_OPERATION]
    id = "Zoom.User.Promoted.to.Privileged.Role-prototype"
    tests = zoom_user_promotedto_privileged_role_tests
    PRIVILEGED_ROLES = ("Admin", "Co-Owner", "Owner", "Billing Admin")

    def extract_values(self, event):
        operator = event.get("operator", "<operator-not-found>")
        operation_detail = event.get("operation_detail", "")
        email = re.search("[\\w.+-c]+@[\\w-]+\\.[\\w.-]+", operation_detail)[0] or "<email-not-found>"
        fromto = re.findall("from ([-\\s\\w]+) to ([-\\s\\w]+)", operation_detail) or [
            ("<from-role-not-found>", "<to-role-not-found>"),
        ]
        from_role, to_role = fromto[0] or ("<role-not-found>", "<role-not-found>")
        return (operator, email, from_role, to_role)

    def rule(self, event):
        if (
            "Update" in event.get("action", "")
            and event.get("category_type") == "User"
            and event.get("operation_detail", "").startswith("Change Role")
        ):
            _, _, from_role, to_role = self.extract_values(event)
            return to_role in self.PRIVILEGED_ROLES and from_role not in self.PRIVILEGED_ROLES
        return False

    def title(self, event):
        operator, email, from_role, to_role = self.extract_values(event)
        return f"Zoom: [{email}]'s role was changed from [{from_role}] to [{to_role}] by [{operator}]."



import argparse
import json
import logging
import os
import tempfile
import time
import zipfile
from dataclasses import asdict
from fnmatch import fnmatch
from typing import Any, Optional, Tuple

from pypanther import cli_output, display, testing
from pypanther.import_main import NoMainModuleError, import_main
from pypanther.registry import registered_rules
from pypanther.vendor.panther_analysis_tool.backend.client import (
    AsyncBulkUploadParams,
    AsyncBulkUploadStatusParams,
    AsyncBulkUploadStatusResponse,
    BackendError,
    BackendResponse,
    BulkUploadMultipartError,
)
from pypanther.vendor.panther_analysis_tool.backend.client import Client as BackendClient
from pypanther.vendor.panther_analysis_tool.util import convert_unicode

INDENT = " " * 2
IGNORE_FOLDERS = [
    ".mypy_cache",
    "pypanther",
    "panther_analysis",
    ".git",
    "__pycache__",
    "tests",
]

UPLOAD_RESULT_SUCCESS = "UPLOAD_SUCCEEDED"
UPLOAD_RESULT_FAILURE = "UPLOAD_FAILED"
UPLOAD_RESULT_TESTS_FAILED = "TESTS_FAILED"


def run(backend: BackendClient, args: argparse.Namespace) -> Tuple[int, str]:
    if not args.confirm:
        err = confirm()
        if err is not None:
            return 0, ""

    try:
        import_main(os.getcwd(), "main")
    except NoMainModuleError:
        logging.error("No main.py found")  # noqa: TRY400
        return 1, ""

    test_results = testing.TestResults()  # default to something, so it can be used below in output
    if not args.skip_tests:
        test_results = testing.run_tests(args)
        if test_results.had_failed_tests():
            if args.output == display.OUTPUT_TYPE_JSON:
                output = get_upload_output_as_dict(
                    None,
                    test_results,
                    [],
                    args.verbose,
                    args.skip_tests,
                    UPLOAD_RESULT_TESTS_FAILED,
                )
                print(json.dumps(output, indent=display.JSON_INDENT_LEVEL))
            return 1, ""

        if args.output == display.OUTPUT_TYPE_TEXT:
            print()  # new line to separate test output from upload output

    if args.verbose and args.output == display.OUTPUT_TYPE_TEXT:
        print_registered_rules()

    with tempfile.NamedTemporaryFile() as tmp:
        zip_info = zip_contents(tmp)

        if args.verbose and args.output == display.OUTPUT_TYPE_TEXT:
            print_included_files(zip_info)

        try:
            upload_stats = upload_zip(
                backend,
                archive=tmp.name,
                verbose=args.verbose,
                output_type=args.output,
                max_retries=args.max_retries,
            )

            if args.output == display.OUTPUT_TYPE_JSON:
                output = get_upload_output_as_dict(
                    upload_stats,
                    test_results,
                    zip_info,
                    args.verbose,
                    args.skip_tests,
                    UPLOAD_RESULT_SUCCESS,
                )
                print(json.dumps(output, indent=display.JSON_INDENT_LEVEL))

        except BackendError as be_err:
            multi_err = BulkUploadMultipartError.from_jsons(convert_unicode(be_err))
            if args.output == display.OUTPUT_TYPE_TEXT:
                print_backend_issues(multi_err)
            elif args.output == display.OUTPUT_TYPE_JSON:
                output = get_failed_upload_as_dict(
                    multi_err,
                    test_results,
                    zip_info,
                    args.verbose,
                    args.skip_tests,
                    UPLOAD_RESULT_FAILURE,
                )
                print(json.dumps(output, indent=display.JSON_INDENT_LEVEL))
            return 1, ""

    return 0, ""


def zip_contents(named_temp_file: Any) -> list[zipfile.ZipInfo]:
    with zipfile.ZipFile(named_temp_file, "w") as zip_out:
        for root, dir_, files in os.walk("."):
            for bad in IGNORE_FOLDERS:
                if bad in dir_:
                    dir_.remove(bad)

            for file in files:
                if not fnmatch(file, "*.py"):
                    continue

                filepath = os.path.join(root, file)

                zip_out.write(
                    filepath,
                    arcname=filepath,
                )

        return zip_out.infolist()


def upload_zip(
    backend: BackendClient,
    archive: str,
    verbose: bool,
    output_type: str,
    max_retries: int = 10,
) -> AsyncBulkUploadStatusResponse:
    # extract max retries we should handle
    if max_retries > 10:
        logging.warning("max_retries cannot be greater than 10, defaulting to 10")
        max_retries = 10
    elif max_retries < 0:
        logging.warning("max_retries cannot be negative, defaulting to 0")
        max_retries = 0

    with open(archive, "rb") as analysis_zip:
        if verbose and output_type == display.OUTPUT_TYPE_TEXT:
            print(cli_output.header("Uploading to Panther"))
        elif output_type == display.OUTPUT_TYPE_TEXT:
            print("Uploading to Panther")
            print()  # new line

        upload_params = AsyncBulkUploadParams(zip_bytes=analysis_zip.read())
        retry_count = 0

    while True:
        try:
            start_upload_response = backend.async_bulk_upload(upload_params)
            if verbose and output_type == display.OUTPUT_TYPE_TEXT:
                print(INDENT, "- Upload started")

            while True:
                time.sleep(2)

                status_response = backend.async_bulk_upload_status(
                    AsyncBulkUploadStatusParams(receipt_id=start_upload_response.data.receipt_id),
                )
                if status_response is not None:
                    if output_type == display.OUTPUT_TYPE_TEXT:
                        if verbose:
                            print(INDENT, "- Upload finished")
                            print()  # new line

                        print_upload_statistics(status_response)

                    if output_type == display.OUTPUT_TYPE_TEXT:
                        print(cli_output.success("Upload succeeded"))

                    return status_response.data

                if verbose and output_type == display.OUTPUT_TYPE_TEXT:
                    print(INDENT, "- Upload still in progress")

        except BackendError as be_err:
            if verbose and output_type == display.OUTPUT_TYPE_TEXT:
                print()  # new line for after upload statuses

            if be_err.permanent is True:
                raise be_err

            if max_retries - retry_count > 0:
                retry_count += 1

                if verbose and output_type == display.OUTPUT_TYPE_TEXT:
                    print(INDENT, "- Upload failed")
                    print(INDENT, f"- Will retry in 30 seconds ({max_retries - retry_count} retries remaining)")

                time.sleep(30)

            else:
                logging.warning("Exhausted retries attempting to perform bulk upload.")
                raise be_err

        # PEP8 guide states it is OK to catch BaseException if you log it.
        except BaseException as err:  # pylint: disable=broad-except
            raise err


def confirm() -> Optional[str]:
    warning_text = cli_output.warning(
        "WARNING: pypanther upload is under active development and not recommended for use"
        " without guidance from the Panther team. Would you like to proceed? [y/n]: ",
    )
    choice = input(warning_text).lower()
    if choice != "y":
        print(cli_output.warning(f'Exiting upload due to entered response "{choice}" which is not "y"'))
        return "User did not confirm"

    print()  # new line
    return None


def get_upload_output_as_dict(
    upload_stats: AsyncBulkUploadStatusResponse | None,
    test_results: testing.TestResults,
    zip_infos: list[zipfile.ZipInfo],
    verbose: bool,
    skip_tests: bool,
    upload_result: str,
) -> dict:
    output: dict[str, Any] = {"result": upload_result}
    if upload_stats is not None:
        output["upload_statistics"] = asdict(upload_stats)
    if not skip_tests:
        output["tests"] = testing.test_output_dict(test_results, verbose)
    if verbose:
        output["registered_rules"] = [rule.id for rule in registered_rules()]
        if len(zip_infos) > 0:
            output["included_files"] = [info.filename for info in zip_infos]

    return output


def get_failed_upload_as_dict(
    err: BulkUploadMultipartError,
    test_results: testing.TestResults,
    zip_infos: list[zipfile.ZipInfo],
    verbose: bool,
    skip_tests: bool,
    upload_result: str,
) -> dict:
    output = {"failed_upload_details": err.asdict(), "result": upload_result}
    if not skip_tests:
        output["tests"] = testing.test_output_dict(test_results, verbose)
    if verbose:
        output["registered_rules"] = [rule.id for rule in registered_rules()]
        if len(zip_infos) > 0:
            output["included_files"] = [info.filename for info in zip_infos]

    return output


def print_registered_rules() -> None:
    if len(registered_rules()) == 0:
        return

    print(cli_output.header("Registered Rules"))
    for i, rule in enumerate(registered_rules(), start=1):
        print(INDENT, f"{i}. {rule.id}")
    print()  # new line


def print_included_files(zip_info: list[zipfile.ZipInfo]) -> None:
    print(cli_output.header("Included files:"))
    for info in zip_info:
        print(INDENT, f"- {info.filename}")
    print()  # new line


def print_upload_statistics(status_response: BackendResponse[AsyncBulkUploadStatusResponse]) -> None:
    print(cli_output.header("Upload Statistics"))
    for category, stats in asdict(status_response.data).items():
        print(INDENT, cli_output.bold(f"{str(category).title()}:"))
        for k, stat in stats.items():
            print(
                INDENT * 2,
                "{:<9} {}".format(str(k).title() + ":", stat),
            )

    print()  # new line


def print_backend_issues(err: BulkUploadMultipartError) -> None:
    print(cli_output.failed("Upload Failed"))
    if err.get_error() != "":
        print(INDENT, f"- {cli_output.failed(err.get_error())}")
    if len(err.get_issues()) > 0:
        for issue in err.get_issues():
            print(INDENT, f"- Error: {cli_output.failed(issue.error_message)}")
            print(INDENT, f"  Path:  {cli_output.failed(issue.path)}")



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

gcp_logging_settings_modified_tests: list[RuleTest] = [
    RuleTest(
        name="Other Event",
        expected_result=False,
        log={
            "insertid": "abcdefghijklmn",
            "logname": "projects/gcp-project1/logs/cloudaudit.googleapis.com%2Factivity",
            "operation": {
                "id": "1234567890123-gcp-project1:abcdefghijklmnopqrstuvwz",
                "last": True,
                "producer": "bigquery.googleapis.com",
            },
            "p_any_emails": ["user@company.io"],
            "p_any_ip_addresses": ["1.2.3.4"],
            "p_event_time": "2023-03-28 18:37:06.079",
            "p_log_type": "GCP.AuditLog",
            "p_parse_time": "2023-03-28 18:38:14.478",
            "p_row_id": "06bf03d9d5dfbadba981899e1787bf05",
            "p_schema_version": 0,
            "p_source_id": "964c7894-9a0d-4ddf-864f-0193438221d6",
            "p_source_label": "gcp-logsource",
            "protopayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {"principalEmail": "user@company.io"},
                "authorizationInfo": [
                    {
                        "granted": True,
                        "permission": "bigquery.tables.delete",
                        "resource": "projects/gcp-project1/datasets/test1/tables/newtable",
                    },
                ],
                "metadata": {
                    "@type": "type.googleapis.com/google.cloud.audit.BigQueryAuditMetadata",
                    "methodName": "google.cloud.bigquery.v2.JobService.InsertJob",
                    "requestMetadata": {
                        "callerIP": "1.2.3.4",
                        "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/111.0.0.0 Safari/537.36,gzip(gfe),gzip(gfe)",
                    },
                    "resourceName": "projects/gcp-project1/datasets/test1/tables/newtable",
                    "serviceName": "bigquery.googleapis.com",
                    "status": {},
                    "tableDeletion": {
                        "jobName": "projects/gcp-project1/jobs/bquxjob_5e4a0679_18729a639d7",
                        "reason": "QUERY",
                    },
                },
                "receivetimestamp": "2023-03-28 18:37:06.745",
                "resource": {
                    "labels": {"dataset_id": "test1", "project_id": "gcp-project1"},
                    "type": "bigquery_dataset",
                },
                "severity": "NOTICE",
                "timestamp": "2023-03-28 18:37:06.079",
            },
        },
    ),
    RuleTest(
        name="Sink Update Event",
        expected_result=True,
        log={
            "insertid": "ezyd47c12y",
            "logname": "projects/gcp-project1/logs/cloudaudit.googleapis.com%2Factivity",
            "p_any_ip_addresses": ["1.2.3.4"],
            "p_event_time": "2023-03-09 16:41:30.524",
            "p_log_type": "GCP.AuditLog",
            "p_parse_time": "2023-03-09 16:44:14.617",
            "p_row_id": "1234567909689348911",
            "p_source_id": "4fc88a5a-2d51-4279-9c4a-08fa7cc52566",
            "p_source_label": "gcplogsource",
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {"principalEmail": "test@company.io"},
                "authorizationInfo": [
                    {
                        "granted": True,
                        "permission": "logging.sinks.update",
                        "resource": "projects/gcp-project1/sinks/log-sink",
                        "resourceAttributes": {
                            "name": "projects/gcp-project1/sinks/log-sink",
                            "service": "logging.googleapis.com",
                        },
                    },
                ],
                "methodName": "google.logging.v2.ConfigServiceV2.UpdateSink",
                "request": {
                    "@type": "type.googleapis.com/google.logging.v2.UpdateSinkRequest",
                    "sink": {
                        "destination": "pubsub.googleapis.com/projects/gcp-project1/topics/gcp-topic1",
                        "exclusions": [{"filter": "protoPayload.serviceName = 'k8s.io", "name": "excludek8s"}],
                        "name": "log-sink",
                        "writerIdentity": "serviceAccount:p197946410614-915152@gcp-sa-logging.iam.gserviceaccount.com",
                    },
                    "sinkName": "projects/gcp-project1/sinks/log-sink",
                    "uniqueWriterIdentity": True,
                    "updateMask": "exclusions",
                },
                "requestMetadata": {
                    "callerIP": "1.2.3.4",
                    "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/110.0.0.0 Safari/537.36,gzip(gfe),gzip(gfe)",
                    "destinationAttributes": {},
                    "requestAttributes": {"auth": {}, "time": "2023-03-09T16:41:30.540045105Z"},
                },
                "resourceName": "projects/gcp-project1/sinks/log-sink",
                "serviceName": "logging.googleapis.com",
                "status": {},
            },
            "receivetimestamp": "2023-03-09 16:41:32.21",
            "resource": {
                "labels": {"destination": "", "name": "log-sink", "project_id": "gcp-project1"},
                "type": "logging_sink",
            },
            "severity": "NOTICE",
            "timestamp": "2023-03-09 16:41:30.524",
        },
    ),
]


@panther_managed
class GCPLoggingSettingsModified(Rule):
    default_description = "Detects any changes made to logging settings"
    display_name = "GCP Logging Settings Modified"
    default_reference = "https://cloud.google.com/logging/docs/default-settings"
    default_severity = Severity.LOW
    log_types = [LogType.GCP_AUDIT_LOG]
    id = "GCP.Logging.Settings.Modified-prototype"
    tests = gcp_logging_settings_modified_tests

    def rule(self, event):
        return all(
            [
                deep_get(event, "protoPayload", "serviceName", default="") == "logging.googleapis.com",
                "Update" in deep_get(event, "protoPayload", "methodName", default=""),
            ],
        )

    def title(self, event):
        resource = deep_get(event, "protoPayload", "resourceName", default="<RESOURCE_NOT_FOUND>")
        actor = deep_get(event, "protoPayload", "authenticationInfo", "principalEmail", default="<ACTOR_NOT_FOUND>")
        return f"GCP [{resource}] logging settings modified by [{actor}]."

    def alert_context(self, event):
        return {
            "resource": deep_get(event, "protoPayload", "resourceName", default="<RESOURCE_NOT_FOUND>"),
            "actor": deep_get(
                event,
                "protoPayload",
                "authenticationInfo",
                "principalEmail",
                default="<ACTOR_NOT_FOUND>",
            ),
            "method": deep_get(event, "protoPayload", "methodName", default="<METHOD_NOT_FOUND>"),
        }



import pypanther.helpers.panther_event_type_helpers as event_type
from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

standard_admin_role_assigned_tests: list[RuleTest] = [
    RuleTest(
        name="GCP - Admin Assigned",
        expected_result=True,
        log={
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "serviceName": "cloudresourcemanager.googleapis.com",
                "methodName": "SetIamPolicy",
                "authenticationInfo": {"principalEmail": "bob@example.com"},
                "requestMetadata": {"callerIP": "4.4.4.4"},
                "serviceData": {
                    "@type": "type.googleapis.com/google.iam.v1.logging.AuditData",
                    "policyDelta": {
                        "bindingDeltas": [
                            {
                                "action": "ADD",
                                "member": "cat@example.com",
                                "role": "roles/resourcemanager.organizationAdmin",
                            },
                        ],
                    },
                },
            },
            "p_log_type": "GCP.AuditLog",
        },
    ),
    RuleTest(
        name="GCP - Multiple Admin Roles Assigned",
        expected_result=True,
        log={
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "serviceName": "cloudresourcemanager.googleapis.com",
                "methodName": "SetIamPolicy",
                "authenticationInfo": {"principalEmail": "bob@example.com"},
                "requestMetadata": {"callerIP": "4.4.4.4"},
                "serviceData": {
                    "@type": "type.googleapis.com/google.iam.v1.logging.AuditData",
                    "policyDelta": {
                        "bindingDeltas": [
                            {
                                "action": "ADD",
                                "member": "cat@example.com",
                                "role": "roles/resourcemanager.organizationAdmin",
                            },
                            {"action": "ADD", "member": "dog@example.com", "role": "roles/owner"},
                        ],
                    },
                },
            },
            "p_log_type": "GCP.AuditLog",
        },
    ),
    RuleTest(
        name="GSuite - Other Admin Action",
        expected_result=False,
        log={
            "actor": {"email": "bobert@example.com"},
            "id": {"applicationName": "admin"},
            "events": [{"type": "DELEGATED_ADMIN_SETTINGS", "name": "RENAME_ROLE"}],
            "p_log_type": "GSuite.Reports",
        },
    ),
    RuleTest(
        name="GSuite - Privileges Assigned",
        expected_result=True,
        log={
            "actor": {"email": "bobert@example.com"},
            "id": {"applicationName": "admin"},
            "events": [
                {
                    "type": "DELEGATED_ADMIN_SETTINGS",
                    "name": "ASSIGN_ROLE",
                    "parameters": [
                        {"name": "ROLE_NAME", "value": "Some Admin Role"},
                        {"name": "USER_EMAIL", "value": "bob@example.com"},
                    ],
                },
            ],
            "p_log_type": "GSuite.Reports",
        },
    ),
    RuleTest(
        name="OneLogin - Non permissions assigned event",
        expected_result=False,
        log={"event_type_id": 8, "p_log_type": "OneLogin.Events"},
    ),
    RuleTest(
        name="OneLogin - Non super user permissions assigned",
        expected_result=False,
        log={"event_type_id": 72, "privilege_name": "Manage users", "p_log_type": "OneLogin.Events"},
    ),
    RuleTest(
        name="OneLogin - Super user permissions assigned",
        expected_result=True,
        log={
            "event_type_id": 72,
            "privilege_name": "Super user",
            "user_name": "Evil Bob",
            "actor_user_name": "Bobert O'Bobly",
            "p_log_type": "OneLogin.Events",
        },
    ),
    RuleTest(
        name="Github - User Promoted",
        expected_result=True,
        log={"actor": "cat", "action": "team.promote_maintainer", "p_log_type": "GitHub.Audit", "user": "bob"},
    ),
    RuleTest(
        name="Github - Admin Added",
        expected_result=True,
        log={"actor": "cat", "action": "business.add_admin", "p_log_type": "GitHub.Audit", "user": "bob"},
    ),
    RuleTest(
        name="Github - Admin Invited",
        expected_result=True,
        log={"actor": "cat", "action": "business.invite_admin", "p_log_type": "GitHub.Audit", "user": "bob"},
    ),
    RuleTest(
        name="Github - Unknown Admin Role",
        expected_result=False,
        log={"actor": "cat", "action": "unknown.admin_role", "p_log_type": "GitHub.Audit", "user": "bob"},
    ),
    RuleTest(
        name="Zendesk - Admin Role Downgraded",
        expected_result=False,
        log={
            "url": "https://myzendek.zendesk.com/api/v2/audit_logs/111222333444.json",
            "id": 123456789123,
            "action_label": "Updated",
            "actor_id": 123,
            "source_id": 123,
            "source_type": "user",
            "source_label": "Bob Cat",
            "action": "update",
            "change_description": "Role changed from Administrator to End User",
            "ip_address": "127.0.0.1",
            "created_at": "2021-05-28T18:39:50Z",
            "p_log_type": "Zendesk.Audit",
        },
    ),
    RuleTest(
        name="Zendesk - Admin Role Assigned",
        expected_result=True,
        log={
            "url": "https://myzendek.zendesk.com/api/v2/audit_logs/111222333444.json",
            "id": 123456789123,
            "action_label": "Updated",
            "actor_id": 123,
            "source_id": 123,
            "source_type": "user",
            "source_label": "Bob Cat",
            "action": "update",
            "change_description": "Role changed from End User to Administrator",
            "ip_address": "127.0.0.1",
            "created_at": "2021-05-28T18:39:50Z",
            "p_log_type": "Zendesk.Audit",
        },
    ),
    RuleTest(
        name="Zendesk - App Admin Role Assigned",
        expected_result=True,
        log={
            "url": "https://myzendek.zendesk.com/api/v2/audit_logs/111222333444.json",
            "id": 123456789123,
            "action_label": "Updated",
            "actor_id": 123,
            "source_id": 123,
            "source_type": "user",
            "source_label": "Bob Cat",
            "action": "update",
            "change_description": "Explore role changed from not set to Admin\nGuide role changed from not set to Admin\nSupport role changed from not set to Admin\nTalk role changed from not set to Admin",
            "ip_address": "127.0.0.1",
            "created_at": "2021-05-28T18:39:50Z",
            "p_log_type": "Zendesk.Audit",
        },
    ),
    RuleTest(
        name="Asana - Normal Login",
        expected_result=False,
        log={
            "actor": {"actor_type": "user", "email": "homer@springfield.com", "gid": "2222222", "name": "Homer"},
            "context": {"client_ip_address": "8.8.8.8", "context_type": "web"},
            "created_at": "2021-10-21T23:38:10.364Z",
            "details": {"method": ["ONE_TIME_KEY"]},
            "event_category": "logins",
            "event_type": "user_login_succeeded",
            "gid": "222222222",
            "resource": {"email": "homer@springfield.com", "gid": "2222222", "name": "homer", "resource_type": "user"},
            "p_log_type": "Asana.Audit",
            "p_parse_time": "2021-06-04 10:02:33.650807",
            "p_event_time": "2021-06-04 09:59:53.650807",
        },
    ),
    RuleTest(
        name="Asana - Admin Added",
        expected_result=True,
        log={
            "actor": {"actor_type": "user", "name": "Homer"},
            "context": {"client_ip_address": "1.1.1.1", "context_type": "web"},
            "created_at": "2021-10-21T23:38:18.319Z",
            "details": {
                "group": {
                    "gid": "11111",
                    "name": "1183399881404774.2lgxga.asanatest1.us",
                    "resource_type": "workspace",
                },
                "new_value": "member",
                "old_value": "super_admin",
            },
            "event_category": "roles",
            "event_type": "user_workspace_admin_role_changed",
            "gid": "22222",
            "resource": {
                "email": "marge@springfield.com",
                "gid": "222222",
                "name": "Marge Simpson",
                "resource_type": "user",
            },
            "p_log_type": "Asana.Audit",
        },
    ),
]


@panther_managed
class StandardAdminRoleAssigned(Rule):
    id = "Standard.AdminRoleAssigned-prototype"
    display_name = "Admin Role Assigned"
    log_types = [
        LogType.ASANA_AUDIT,
        LogType.ATLASSIAN_AUDIT,
        LogType.GCP_AUDIT_LOG,
        LogType.GITHUB_AUDIT,
        LogType.GSUITE_REPORTS,
        LogType.ONELOGIN_EVENTS,
        LogType.ZENDESK_AUDIT,
    ]
    tags = ["DataModel", "Privilege Escalation:Valid Accounts"]
    default_severity = Severity.MEDIUM
    reports = {"MITRE ATT&CK": ["TA0004:T1078"]}
    default_description = "Assigning an admin role manually could be a sign of privilege escalation"
    default_runbook = "Verify with the user who attached the role or add to a allowlist"
    default_reference = "https://medium.com/@gokulelango1040/privilege-escalation-attacks-28a9ef226abb"
    summary_attributes = ["p_any_ip_addresses"]
    tests = standard_admin_role_assigned_tests

    def rule(self, event):
        # filter events on unified data model field
        return event.udm("event_type") == event_type.ADMIN_ROLE_ASSIGNED

    def title(self, event):
        # use unified data model field in title
        return f"{event.get('p_log_type')}: [{event.udm('actor_user')}] assigned admin privileges [{event.udm('assigned_admin_role')}] to [{event.udm('user')}]"

    def alert_context(self, event):
        return {"ips": event.get("p_any_ip_addresses", []), "actor": event.udm("actor_user"), "user": event.udm("user")}



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

push_security_open_security_finding_tests: list[RuleTest] = [
    RuleTest(
        name="Resolved Finding",
        expected_result=False,
        log={
            "id": "c478966c-f927-411c-b919-179832d3d50c",
            "new": {
                "accountId": None,
                "appId": "2a2197de-ad2c-47e4-8dcb-fb0f04cf83e0",
                "appType": "PUSH_SECURITY",
                "creationTimestamp": 1698064423.0,
                "employeeId": "379ac7ea-ff2a-42ef-af37-06d2020dc46a",
                "id": "d6a32ba5-0532-4a66-8137-48cdf409c972",
                "passwordId": "c4a045a1-5331-4714-af83-6a361e98960d",
                "state": "RESOLVED",
                "type": "WEAK_PASSWORD",
            },
            "object": "FINDING",
            "old": {
                "accountId": None,
                "appId": "2a2197de-ad2c-47e4-8dcb-fb0f04cf83e0",
                "appType": "PUSH_SECURITY",
                "creationTimestamp": 1698064423.0,
                "employeeId": "379ac7ea-ff2a-42ef-af37-06d2020dc46a",
                "id": "d6a32ba5-0532-4a66-8137-48cdf409c972",
                "passwordId": "c4a045a1-5331-4714-af83-6a361e98960d",
                "state": "OPEN",
                "type": "WEAK_PASSWORD",
            },
            "timestamp": 1698604061.0,
            "type": "UPDATE",
            "version": "1",
        },
    ),
    RuleTest(
        name="New Finding",
        expected_result=True,
        log={
            "id": "c478966c-f927-411c-b919-179832d3d50c",
            "new": {
                "accountId": None,
                "appId": "2a2197de-ad2c-47e4-8dcb-fb0f04cf83e0",
                "appType": "PUSH_SECURITY",
                "creationTimestamp": 1698064423.0,
                "employeeId": "379ac7ea-ff2a-42ef-af37-06d2020dc46a",
                "id": "d6a32ba5-0532-4a66-8137-48cdf409c972",
                "passwordId": "c4a045a1-5331-4714-af83-6a361e98960d",
                "state": "OPEN",
                "type": "WEAK_PASSWORD",
            },
            "object": "FINDING",
            "old": None,
            "timestamp": 1698604061.0,
            "type": "CREATE",
            "version": "1",
        },
    ),
    RuleTest(
        name="Reopened Finding",
        expected_result=True,
        log={
            "id": "c478966c-f927-411c-b919-179832d3d50c",
            "new": {
                "accountId": None,
                "appId": "2a2197de-ad2c-47e4-8dcb-fb0f04cf83e0",
                "appType": "PUSH_SECURITY",
                "creationTimestamp": 1698064423.0,
                "employeeId": "379ac7ea-ff2a-42ef-af37-06d2020dc46a",
                "id": "d6a32ba5-0532-4a66-8137-48cdf409c972",
                "passwordId": "c4a045a1-5331-4714-af83-6a361e98960d",
                "state": "OPEN",
                "type": "WEAK_PASSWORD",
            },
            "object": "FINDING",
            "old": {
                "accountId": None,
                "appId": "2a2197de-ad2c-47e4-8dcb-fb0f04cf83e0",
                "appType": "PUSH_SECURITY",
                "creationTimestamp": 1698064423.0,
                "employeeId": "379ac7ea-ff2a-42ef-af37-06d2020dc46a",
                "id": "d6a32ba5-0532-4a66-8137-48cdf409c972",
                "passwordId": "c4a045a1-5331-4714-af83-6a361e98960d",
                "state": "RESOLVED",
                "type": "WEAK_PASSWORD",
            },
            "timestamp": 1698604061.0,
            "type": "UPDATE",
            "version": "1",
        },
    ),
]


@panther_managed
class PushSecurityOpenSecurityFinding(Rule):
    id = "Push.Security.Open.Security.Finding-prototype"
    display_name = "Push Security Open Security Finding"
    log_types = [LogType.PUSH_SECURITY_ENTITIES]
    default_severity = Severity.INFO
    tests = push_security_open_security_finding_tests

    def rule(self, event):
        if event.get("object") != "FINDING":
            return False
        event_type = event.get("type")
        if event_type == "CREATE":
            return True
        if event_type == "UPDATE" and event.deep_get("new", "state") == "OPEN":
            return True
        return False

    def title(self, event):
        new_type = event.deep_get("new", "type")
        app_type = event.deep_get("new", "appType")
        return f"Open finding {new_type} for app {app_type}"



from pypanther.base import DataModel, DataModelMapping, LogType
from pypanther.helpers.panther_base_helpers import deep_get


def get_dns_query(event):
    # Strip trailing period.
    # Domain Names from Crowdstrike FDR end with a trailing period, such as google.com.
    domain = deep_get(event, "event", "DomainName", default=None)
    if domain:
        domain = domain.rstrip(".")
    return domain


def get_process_name(event):
    platform = event.get("event_platform")
    # Extract process name from path
    # Win = \Device\HarddiskVolume2\Windows\System32\winlogon.exe
    # Lin = /usr/bin/run-parts
    # Mac = /usr/libexec/xpcproxy
    image_fn = deep_get(event, "event", "ImageFileName")
    if not image_fn:
        return None  # Explicitly return None if the key DNE
    if platform == "Win":
        return image_fn.split("\\")[-1]
    return image_fn.split("/")[-1]


class StandardCrowdstrikeFDR(DataModel):
    id: str = "Standard.Crowdstrike.FDR"
    display_name: str = "Crowdstrike FDR"
    enabled: bool = True
    log_types: list[str] = [LogType.CROWDSTRIKE_FDR_EVENT]
    mappings: list[DataModelMapping] = [
        DataModelMapping(name="actor_user", path="$.event.UserName"),
        DataModelMapping(name="cmd", path="$.event.CommandLine"),
        DataModelMapping(name="destination_ip", path="$.event.RemoteAddressIP4"),
        DataModelMapping(name="destination_port", path="$.event.RemotePort"),
        DataModelMapping(name="dns_query", method=get_dns_query),
        DataModelMapping(name="process_name", method=get_process_name),
        DataModelMapping(name="source_ip", path="$.aip"),
        DataModelMapping(name="source_port", path="$.event.LocalPort"),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context, deep_get

awsrds_public_restore_tests: list[RuleTest] = [
    RuleTest(
        name="Not-Restore-RDS-Request",
        expected_result=False,
        log={
            "awsRegion": "us-east-1",
            "eventID": "797163d3-5726-441d-80a7-6eeb7464acd4",
            "eventName": "CreateDBInstance",
            "eventSource": "rds.amazonaws.com",
            "eventTime": "2018-07-30T22:14:06Z",
            "eventType": "AwsApiCall",
            "eventVersion": "1.04",
            "recipientAccountId": "123456789012",
            "requestID": "daf2e3f5-96a3-4df7-a026-863f96db793e",
            "requestParameters": {
                "allocatedStorage": 20,
                "dBInstanceClass": "db.m1.small",
                "dBInstanceIdentifier": "test-instance",
                "enableCloudwatchLogsExports": ["audit", "error", "general", "slowquery"],
                "engine": "mysql",
                "masterUserPassword": "****",
                "masterUsername": "myawsuser",
            },
            "responseElements": {
                "allocatedStorage": 20,
                "autoMinorVersionUpgrade": True,
                "backupRetentionPeriod": 1,
                "cACertificateIdentifier": "rds-ca-2015",
                "copyTagsToSnapshot": False,
                "dBInstanceArn": "arn:aws:rds:us-east-1:123456789012:db:test-instance",
                "dBInstanceClass": "db.m1.small",
                "dBInstanceIdentifier": "test-instance",
                "dBInstanceStatus": "creating",
                "dBParameterGroups": [{"dBParameterGroupName": "default.mysql8.0", "parameterApplyStatus": "in-sync"}],
                "dBSecurityGroups": [],
                "dBSubnetGroup": {
                    "dBSubnetGroupDescription": "default",
                    "dBSubnetGroupName": "default",
                    "subnetGroupStatus": "Complete",
                    "subnets": [
                        {
                            "subnetAvailabilityZone": {"name": "us-east-1b"},
                            "subnetIdentifier": "subnet-cbfff283",
                            "subnetStatus": "Active",
                        },
                        {
                            "subnetAvailabilityZone": {"name": "us-east-1e"},
                            "subnetIdentifier": "subnet-d7c825e8",
                            "subnetStatus": "Active",
                        },
                        {
                            "subnetAvailabilityZone": {"name": "us-east-1f"},
                            "subnetIdentifier": "subnet-6746046b",
                            "subnetStatus": "Active",
                        },
                        {
                            "subnetAvailabilityZone": {"name": "us-east-1c"},
                            "subnetIdentifier": "subnet-bac383e0",
                            "subnetStatus": "Active",
                        },
                        {
                            "subnetAvailabilityZone": {"name": "us-east-1d"},
                            "subnetIdentifier": "subnet-42599426",
                            "subnetStatus": "Active",
                        },
                        {
                            "subnetAvailabilityZone": {"name": "us-east-1a"},
                            "subnetIdentifier": "subnet-da327bf6",
                            "subnetStatus": "Active",
                        },
                    ],
                    "vpcId": "vpc-136a4c6a",
                },
                "dbInstancePort": 0,
                "dbiResourceId": "db-ETDZIIXHEWY5N7GXVC4SH7H5IA",
                "domainMemberships": [],
                "engine": "mysql",
                "engineVersion": "8.0.28",
                "iAMDatabaseAuthenticationEnabled": False,
                "licenseModel": "general-public-license",
                "masterUsername": "myawsuser",
                "monitoringInterval": 0,
                "multiAZ": False,
                "optionGroupMemberships": [{"optionGroupName": "default:mysql-8-0", "status": "in-sync"}],
                "pendingModifiedValues": {
                    "masterUserPassword": "****",
                    "pendingCloudwatchLogsExports": {"logTypesToEnable": ["audit", "error", "general", "slowquery"]},
                },
                "performanceInsightsEnabled": False,
                "preferredBackupWindow": "10:27-10:57",
                "preferredMaintenanceWindow": "sat:05:47-sat:06:17",
                "publiclyAccessible": True,
                "readReplicaDBInstanceIdentifiers": [],
                "storageEncrypted": False,
                "storageType": "standard",
                "vpcSecurityGroups": [{"status": "active", "vpcSecurityGroupId": "sg-f839b688"}],
            },
            "sourceIPAddress": "192.0.2.0",
            "userAgent": "aws-cli/1.15.42 Python/3.6.1 Darwin/17.7.0 botocore/1.10.42",
            "userIdentity": {
                "accessKeyId": "AKIAI44QH8DHBEXAMPLE",
                "accountId": "123456789012",
                "arn": "arn:aws:iam::123456789012:user/johndoe",
                "principalId": "AKIAIOSFODNN7EXAMPLE",
                "type": "IAMUser",
                "userName": "johndoe",
            },
        },
    ),
    RuleTest(
        name="RDS-Restore-Not-Public",
        expected_result=False,
        log={
            "awsRegion": "us-east-1",
            "eventID": "797163d3-5726-441d-80a7-6eeb7464acd4",
            "eventName": "RestoreDBInstanceFromDBSnapshot",
            "eventSource": "rds.amazonaws.com",
            "eventTime": "2018-07-30T22:14:06Z",
            "eventType": "AwsApiCall",
            "eventVersion": "1.04",
            "recipientAccountId": "123456789012",
            "requestID": "daf2e3f5-96a3-4df7-a026-863f96db793e",
            "requestParameters": {
                "allocatedStorage": 20,
                "dBInstanceClass": "db.m1.small",
                "dBInstanceIdentifier": "test-instance",
                "enableCloudwatchLogsExports": ["audit", "error", "general", "slowquery"],
                "engine": "mysql",
                "masterUserPassword": "****",
                "masterUsername": "myawsuser",
            },
            "responseElements": {
                "allocatedStorage": 20,
                "autoMinorVersionUpgrade": True,
                "backupRetentionPeriod": 1,
                "cACertificateIdentifier": "rds-ca-2015",
                "copyTagsToSnapshot": False,
                "dBInstanceArn": "arn:aws:rds:us-east-1:123456789012:db:test-instance",
                "dBInstanceClass": "db.m1.small",
                "dBInstanceIdentifier": "test-instance",
                "dBInstanceStatus": "creating",
                "dBParameterGroups": [{"dBParameterGroupName": "default.mysql8.0", "parameterApplyStatus": "in-sync"}],
                "dBSecurityGroups": [],
                "dBSubnetGroup": {
                    "dBSubnetGroupDescription": "default",
                    "dBSubnetGroupName": "default",
                    "subnetGroupStatus": "Complete",
                    "subnets": [
                        {
                            "subnetAvailabilityZone": {"name": "us-east-1b"},
                            "subnetIdentifier": "subnet-cbfff283",
                            "subnetStatus": "Active",
                        },
                        {
                            "subnetAvailabilityZone": {"name": "us-east-1e"},
                            "subnetIdentifier": "subnet-d7c825e8",
                            "subnetStatus": "Active",
                        },
                        {
                            "subnetAvailabilityZone": {"name": "us-east-1f"},
                            "subnetIdentifier": "subnet-6746046b",
                            "subnetStatus": "Active",
                        },
                        {
                            "subnetAvailabilityZone": {"name": "us-east-1c"},
                            "subnetIdentifier": "subnet-bac383e0",
                            "subnetStatus": "Active",
                        },
                        {
                            "subnetAvailabilityZone": {"name": "us-east-1d"},
                            "subnetIdentifier": "subnet-42599426",
                            "subnetStatus": "Active",
                        },
                        {
                            "subnetAvailabilityZone": {"name": "us-east-1a"},
                            "subnetIdentifier": "subnet-da327bf6",
                            "subnetStatus": "Active",
                        },
                    ],
                    "vpcId": "vpc-136a4c6a",
                },
                "dbInstancePort": 0,
                "dbiResourceId": "db-ETDZIIXHEWY5N7GXVC4SH7H5IA",
                "domainMemberships": [],
                "engine": "mysql",
                "engineVersion": "8.0.28",
                "iAMDatabaseAuthenticationEnabled": False,
                "licenseModel": "general-public-license",
                "masterUsername": "myawsuser",
                "monitoringInterval": 0,
                "multiAZ": False,
                "optionGroupMemberships": [{"optionGroupName": "default:mysql-8-0", "status": "in-sync"}],
                "pendingModifiedValues": {
                    "masterUserPassword": "****",
                    "pendingCloudwatchLogsExports": {"logTypesToEnable": ["audit", "error", "general", "slowquery"]},
                },
                "performanceInsightsEnabled": False,
                "preferredBackupWindow": "10:27-10:57",
                "preferredMaintenanceWindow": "sat:05:47-sat:06:17",
                "publiclyAccessible": False,
                "readReplicaDBInstanceIdentifiers": [],
                "storageEncrypted": False,
                "storageType": "standard",
                "vpcSecurityGroups": [{"status": "active", "vpcSecurityGroupId": "sg-f839b688"}],
            },
            "sourceIPAddress": "192.0.2.0",
            "userAgent": "aws-cli/1.15.42 Python/3.6.1 Darwin/17.7.0 botocore/1.10.42",
            "userIdentity": {
                "accessKeyId": "AKIAI44QH8DHBEXAMPLE",
                "accountId": "123456789012",
                "arn": "arn:aws:iam::123456789012:user/johndoe",
                "principalId": "AKIAIOSFODNN7EXAMPLE",
                "type": "IAMUser",
                "userName": "johndoe",
            },
        },
    ),
    RuleTest(
        name="RDS-Restore-Public",
        expected_result=True,
        log={
            "awsRegion": "us-east-1",
            "eventID": "797163d3-5726-441d-80a7-6eeb7464acd4",
            "eventName": "RestoreDBInstanceFromDBSnapshot",
            "eventSource": "rds.amazonaws.com",
            "eventTime": "2018-07-30T22:14:06Z",
            "eventType": "AwsApiCall",
            "eventVersion": "1.04",
            "recipientAccountId": "123456789012",
            "requestID": "daf2e3f5-96a3-4df7-a026-863f96db793e",
            "requestParameters": {
                "allocatedStorage": 20,
                "dBInstanceClass": "db.m1.small",
                "dBInstanceIdentifier": "test-instance",
                "enableCloudwatchLogsExports": ["audit", "error", "general", "slowquery"],
                "engine": "mysql",
                "masterUserPassword": "****",
                "masterUsername": "myawsuser",
            },
            "responseElements": {
                "allocatedStorage": 20,
                "autoMinorVersionUpgrade": True,
                "backupRetentionPeriod": 1,
                "cACertificateIdentifier": "rds-ca-2015",
                "copyTagsToSnapshot": False,
                "dBInstanceArn": "arn:aws:rds:us-east-1:123456789012:db:test-instance",
                "dBInstanceClass": "db.m1.small",
                "dBInstanceIdentifier": "test-instance",
                "dBInstanceStatus": "creating",
                "dBParameterGroups": [{"dBParameterGroupName": "default.mysql8.0", "parameterApplyStatus": "in-sync"}],
                "dBSecurityGroups": [],
                "dBSubnetGroup": {
                    "dBSubnetGroupDescription": "default",
                    "dBSubnetGroupName": "default",
                    "subnetGroupStatus": "Complete",
                    "subnets": [
                        {
                            "subnetAvailabilityZone": {"name": "us-east-1b"},
                            "subnetIdentifier": "subnet-cbfff283",
                            "subnetStatus": "Active",
                        },
                        {
                            "subnetAvailabilityZone": {"name": "us-east-1e"},
                            "subnetIdentifier": "subnet-d7c825e8",
                            "subnetStatus": "Active",
                        },
                        {
                            "subnetAvailabilityZone": {"name": "us-east-1f"},
                            "subnetIdentifier": "subnet-6746046b",
                            "subnetStatus": "Active",
                        },
                        {
                            "subnetAvailabilityZone": {"name": "us-east-1c"},
                            "subnetIdentifier": "subnet-bac383e0",
                            "subnetStatus": "Active",
                        },
                        {
                            "subnetAvailabilityZone": {"name": "us-east-1d"},
                            "subnetIdentifier": "subnet-42599426",
                            "subnetStatus": "Active",
                        },
                        {
                            "subnetAvailabilityZone": {"name": "us-east-1a"},
                            "subnetIdentifier": "subnet-da327bf6",
                            "subnetStatus": "Active",
                        },
                    ],
                    "vpcId": "vpc-136a4c6a",
                },
                "dbInstancePort": 0,
                "dbiResourceId": "db-ETDZIIXHEWY5N7GXVC4SH7H5IA",
                "domainMemberships": [],
                "engine": "mysql",
                "engineVersion": "8.0.28",
                "iAMDatabaseAuthenticationEnabled": False,
                "licenseModel": "general-public-license",
                "masterUsername": "myawsuser",
                "monitoringInterval": 0,
                "multiAZ": False,
                "optionGroupMemberships": [{"optionGroupName": "default:mysql-8-0", "status": "in-sync"}],
                "pendingModifiedValues": {
                    "masterUserPassword": "****",
                    "pendingCloudwatchLogsExports": {"logTypesToEnable": ["audit", "error", "general", "slowquery"]},
                },
                "performanceInsightsEnabled": False,
                "preferredBackupWindow": "10:27-10:57",
                "preferredMaintenanceWindow": "sat:05:47-sat:06:17",
                "publiclyAccessible": True,
                "readReplicaDBInstanceIdentifiers": [],
                "storageEncrypted": False,
                "storageType": "standard",
                "vpcSecurityGroups": [{"status": "active", "vpcSecurityGroupId": "sg-f839b688"}],
            },
            "sourceIPAddress": "192.0.2.0",
            "userAgent": "aws-cli/1.15.42 Python/3.6.1 Darwin/17.7.0 botocore/1.10.42",
            "userIdentity": {
                "accessKeyId": "AKIAI44QH8DHBEXAMPLE",
                "accountId": "123456789012",
                "arn": "arn:aws:iam::123456789012:user/johndoe",
                "principalId": "AKIAIOSFODNN7EXAMPLE",
                "type": "IAMUser",
                "userName": "johndoe",
            },
        },
    ),
]


@panther_managed
class AWSRDSPublicRestore(Rule):
    default_description = (
        "Detects the recovery of a new public database instance from a snapshot. It may be part of data exfiltration."
    )
    display_name = "AWS Public RDS Restore"
    reports = {"MITRE ATT&CK": ["TA0010:T1020"]}
    default_reference = "https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_RestoreFromSnapshot.html"
    default_severity = Severity.HIGH
    log_types = [LogType.AWS_CLOUDTRAIL]
    id = "AWS.RDS.PublicRestore-prototype"
    tests = awsrds_public_restore_tests

    def rule(self, event):
        if (
            event.get("eventSource", "") == "rds.amazonaws.com"
            and event.get("eventName", "") == "RestoreDBInstanceFromDBSnapshot"
        ):
            if deep_get(event, "responseElements", "publiclyAccessible"):
                return True
        return False

    def title(self, event):
        return f"Publicly Accessible RDS restore created in [{event.get('recipientAccountId', '')}]"

    def alert_context(self, event):
        return aws_rule_context(event)



from fnmatch import fnmatch

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context

awss3_server_access_unknown_requester_tests: list[RuleTest] = [
    RuleTest(
        name="Expected Access",
        expected_result=False,
        log={
            "bucketowner": "f16a9e81a6589df1c902c86f7982fd14a88787db",
            "bucket": "panther-bootstrap-processeddata-AF1341JAK",
            "time": "2020-02-14 00:53:48.000000000",
            "remoteip": "127.0.0.1",
            "requester": "arn:aws:sts::123456789012:assumed-role/panther-log-analysis-AthenaApiFunctionRole-1KK31J1/panther-athena-api",
            "requestid": "101B7403B9828743",
            "operation": "REST.GET.OBJECT",
            "key": "AWSLogs/o-wwwwwwgggg/234567890123/CloudTrail-Digest/ca-central-1/2020/02/14/234567890123_CloudTrail-Digest_ca-central-1_POrgTrail_us-east-1_20200214T001007Z.json.gz",
            "requesturi": "PUT /AWSLogs/o-wwwwwwgggg/234567890123/CloudTrail-Digest/ca-central-1/2020/02/14/234567890123_CloudTrail-Digest_ca-central-1_POrgTrail_us-east-1_20200214T001007Z.json.gz HTTP/1.1",
            "httpstatus": 200,
            "objectsize": 747,
            "totaltime": 110,
            "turnaroundtime": 20,
            "useragent": "aws-internal/3 aws-sdk-java/1.11.714 Linux/4.9.184-0.1.ac.235.83.329.metal1.x86_64 OpenJDK_64-Bit_Server_VM/25.242-b08 java/1.8.0_242 vendor/Oracle_Corporation",
            "hostid": "neRpT/AXRsS3LMBqq/wND59opwPRWWKn7F6evEhdbS99me5fyIXpVI/MMIn6ECgU1YZAqwuF8Bw=",
            "signatureversion": "SigV4",
            "ciphersuite": "ECDHE-RSA-AES128-SHA",
            "authenticationtype": "AuthHeader",
            "hostheader": "cloudtrail.s3.us-east-1.amazonaws.com",
            "tlsVersion": "TLSv1.2",
        },
    ),
    RuleTest(
        name="Unexpected Access",
        expected_result=True,
        log={
            "bucketowner": "f16a9e81a6589df1c902c86f7982fd14a88787db",
            "bucket": "panther-bootstrap-processeddata-AF1341JAK",
            "time": "2020-02-14 00:53:48.000000000",
            "remoteip": "127.0.0.1",
            "requester": "arn:aws:iam::123456789012:user/jim-bob",
            "requestid": "101B7403B9828743",
            "operation": "REST.GET.OBJECT",
            "key": "AWSLogs/o-wwwwwwgggg/234567890123/CloudTrail-Digest/ca-central-1/2020/02/14/234567890123_CloudTrail-Digest_ca-central-1_POrgTrail_us-east-1_20200214T001007Z.json.gz",
            "requesturi": "PUT /AWSLogs/o-wwwwwwgggg/234567890123/CloudTrail-Digest/ca-central-1/2020/02/14/234567890123_CloudTrail-Digest_ca-central-1_POrgTrail_us-east-1_20200214T001007Z.json.gz HTTP/1.1",
            "httpstatus": 200,
            "objectsize": 747,
            "totaltime": 110,
            "turnaroundtime": 20,
            "useragent": "aws-internal/3 aws-sdk-java/1.11.714 Linux/4.9.184-0.1.ac.235.83.329.metal1.x86_64 OpenJDK_64-Bit_Server_VM/25.242-b08 java/1.8.0_242 vendor/Oracle_Corporation",
            "hostid": "neRpT/AXRsS3LMBqq/wND59opwPRWWKn7F6evEhdbS99me5fyIXpVI/MMIn6ECgU1YZAqwuF8Bw=",
            "signatureversion": "SigV4",
            "ciphersuite": "ECDHE-RSA-AES128-SHA",
            "authenticationtype": "AuthHeader",
            "hostheader": "cloudtrail.s3.us-east-1.amazonaws.com",
            "tlsVersion": "TLSv1.2",
        },
    ),
    RuleTest(
        name="Failed Request",
        expected_result=False,
        log={
            "bucketowner": "f16a9e81a6589df1c902c86f7982fd14a88787db",
            "bucket": "panther-bootstrap-processeddata-AF1341JAK",
            "time": "2020-02-14 00:53:48.000000000",
            "errorcode": "AuthorizationHeaderMalformed",
            "remoteip": "127.0.0.1",
            "requestid": "101B7403B9828743",
            "operation": "REST.GET.OBJECT",
            "key": "AWSLogs/o-wwwwwwgggg/234567890123/CloudTrail-Digest/ca-central-1/2020/02/14/234567890123_CloudTrail-Digest_ca-central-1_POrgTrail_us-east-1_20200214T001007Z.json.gz",
            "requesturi": "PUT /AWSLogs/o-wwwwwwgggg/234567890123/CloudTrail-Digest/ca-central-1/2020/02/14/234567890123_CloudTrail-Digest_ca-central-1_POrgTrail_us-east-1_20200214T001007Z.json.gz HTTP/1.1",
            "httpstatus": 400,
            "objectsize": 747,
            "totaltime": 110,
            "turnaroundtime": 20,
            "useragent": "aws-internal/3 aws-sdk-java/1.11.714 Linux/4.9.184-0.1.ac.235.83.329.metal1.x86_64 OpenJDK_64-Bit_Server_VM/25.242-b08 java/1.8.0_242 vendor/Oracle_Corporation",
            "hostid": "neRpT/AXRsS3LMBqq/wND59opwPRWWKn7F6evEhdbS99me5fyIXpVI/MMIn6ECgU1YZAqwuF8Bw=",
            "signatureversion": "SigV4",
            "ciphersuite": "ECDHE-RSA-AES128-SHA",
            "authenticationtype": "AuthHeader",
            "hostheader": "cloudtrail.s3.us-east-1.amazonaws.com",
            "tlsVersion": "TLSv1.2",
        },
    ),
    RuleTest(
        name="Snowflake Request",
        expected_result=False,
        log={
            "authenticationtype": "AuthHeader",
            "bucket": "panther-bootstrap-processeddata-AF1341JAK",
            "bucketowner": "f16a9e81a6589df1c902c86f7982fd14a88787db",
            "ciphersuite": "ECDHE-RSA-AES128-GCM-SHA256",
            "httpstatus": 200,
            "key": "logs/logdir/year%253D2020/month%253D09/day%253D30/hour%253D19/file.json.gz",
            "requesturi": "GET /logs/logdir/year%3D2020/month%3D09/day%3D30/hour%3D19/file.json.gz HTTP/1.1",
            "objectsize": 4063,
            "operation": "REST.GET.OBJECT",
            "remoteip": "127.0.0.1",
            "requester": "arn:aws:sts::123456789012:assumed-role/panther-snowflake-logprocessing-role-us-west-2/snowflake",
            "requestid": "101B7403B9828743",
            "signatureversion": "SigV4",
            "time": "2020-09-30 20:49:19.000000000",
            "tlsVersion": "TLSv1.2",
            "totaltime": 10,
            "turnaroundtime": 9,
            "useragent": "snowflake/1.0",
        },
    ),
]


@panther_managed
class AWSS3ServerAccessUnknownRequester(Rule):
    id = "AWS.S3.ServerAccess.UnknownRequester-prototype"
    display_name = "AWS S3 Unknown Requester"
    enabled = False
    log_types = [LogType.AWS_S3_SERVER_ACCESS]
    tags = ["AWS", "Configuration Required", "Security Control", "Collection:Data From Cloud Storage Object"]
    reports = {"Panther": ["Data Access"], "MITRE ATT&CK": ["TA0009:T1530"]}
    default_severity = Severity.LOW
    default_description = "Validates that proper IAM entities are accessing sensitive data buckets."
    default_runbook = "If the S3 access is not expected for this bucket, investigate the requester's other traffic."
    default_reference = "https://docs.aws.amazon.com/AmazonS3/latest/userguide/walkthrough1.html"
    summary_attributes = [
        "bucket",
        "key",
        "operation",
        "userAgent",
        "remoteip",
        "requester",
        "p_any_aws_arns",
        "p_any_aws_account_ids",
    ]
    tests = awss3_server_access_unknown_requester_tests
    # pylint: disable=line-too-long
    BUCKET_ROLE_MAPPING = {
        "panther-bootstrap-processeddata-*": [
            "arn:aws:sts::*:assumed-role/panther-cloud-security-EventProcessorFunctionRole-*/panther-aws-event-processor",
            "arn:aws:sts::*:assumed-role/panther-log-analysis-AthenaApiFunctionRole-*/panther-athena-api",
            "arn:aws:sts::*:assumed-role/panther-log-analysis-RulesEngineFunctionRole-*/panther-rules-engine",
            "arn:aws:sts::*:assumed-role/panther-snowflake-logprocessing-role-*/snowflake",
            "arn:aws:sts::*:assumed-role/panther-data-replication-role-*/s3-replication",
        ],
    }
    # pylint: enable=line-too-long

    def _unknown_requester_access(self, event):
        for bucket_pattern, role_patterns in self.BUCKET_ROLE_MAPPING.items():
            if not fnmatch(event.get("bucket", ""), bucket_pattern):
                continue
            if not any(fnmatch(event.get("requester", ""), role_pattern) for role_pattern in role_patterns):
                return True
        return False

    def rule(self, event):
        if event.get("errorcode"):
            return False
        return event.get("operation") == "REST.GET.OBJECT" and self._unknown_requester_access(event)

    def title(self, event):
        return f"Unknown requester accessing data from S3 Bucket [{event.get('bucket', '<UNKNOWN_BUCKET>')}]"

    def alert_context(self, event):
        return aws_rule_context(event)



from fnmatch import fnmatch

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

osquery_mac_osx_attacks_keyboard_events_tests: list[RuleTest] = [
    RuleTest(
        name="App running on Desktop that is watching keyboard events",
        expected_result=True,
        log={
            "name": "pack_osx-attacks_Keyboard_Event_Taps",
            "action": "added",
            "hostIdentifier": "test-host",
            "columns": {"path": "/Users/johnny/Desktop/Siri.app/Contents/MacOS/Siri", "pid": 100, "name": "Siri"},
        },
    ),
    RuleTest(
        name="App is running from approved path",
        expected_result=False,
        log={
            "name": "pack_osx-attacks_Keyboard_Event_Taps",
            "action": "added",
            "hostIdentifier": "test-host",
            "columns": {
                "path": "/System/Library/CoreServices/Siri.app/Contents/MacOS/Siri",
                "pid": 100,
                "name": "Siri",
            },
        },
    ),
    RuleTest(
        name="Unrelated query does not alert",
        expected_result=False,
        log={
            "action": "added",
            "calendarTime": "2020-04-10 23:26:11.000000000",
            "columns": {
                "blocks_size": "4096",
                "inodes": "2448101320",
                "path": "/",
                "blocks": "61202533",
                "blocks_available": "22755926",
                "blocks_free": "58479522",
                "device": "/dev/disk1s5",
                "device_alias": "/dev/disk1s5",
                "flags": "75550721",
                "inodes_free": "2447613763",
                "type": "apfs",
            },
            "counter": 28,
            "decorations": {"host_uuid": "0ec3540f-1dd9-4462-bd28-0f63b2611621", "hostname": "MacBook-Pro.local"},
            "epoch": 0,
            "hostIdentifier": "MacBook-Pro.local",
            "name": "pack/incident-response/mounts",
            "unixTime": 1586561171,
        },
    ),
]


@panther_managed
class OsqueryMacOSXAttacksKeyboardEvents(Rule):
    id = "Osquery.Mac.OSXAttacksKeyboardEvents-prototype"
    display_name = "MacOS Keyboard Events"
    log_types = [LogType.OSQUERY_DIFFERENTIAL]
    tags = ["Osquery", "MacOS", "Malware", "Collection:Input Capture"]
    reports = {"MITRE ATT&CK": ["TA0009:T1056"]}
    default_severity = Severity.MEDIUM
    default_description = "A Key Logger has potentially been detected on a macOS system"
    default_runbook = "Verify the Application monitoring the keyboard taps"
    default_reference = "https://support.apple.com/en-us/HT204899"
    summary_attributes = ["name", "hostIdentifier", "action"]
    tests = osquery_mac_osx_attacks_keyboard_events_tests
    # sip protects against writing malware into the paths below.
    # additional apps can be added to this list based on your environments.
    #
    # more info: https://support.apple.com/en-us/HT204899
    APPROVED_PROCESS_PATHS = {"/System/*", "/usr/*", "/bin/*", "/sbin/*", "/var/*"}
    APPROVED_APPLICATION_NAMES = {"Adobe Photoshop CC 2019"}

    def rule(self, event):
        if "Keyboard_Event_Taps" not in event.get("name", ""):
            return False
        if event.get("action") != "added":
            return False
        process_path = deep_get(event, "columns", "path", default="")
        if process_path == "":
            return False
        if deep_get(event, "columns", "name") in self.APPROVED_APPLICATION_NAMES:
            return False
        # Alert if the process is running outside any of the approved paths
        # TODO: Convert this fnmatch pattern below to a helper
        return not any(fnmatch(process_path, p) for p in self.APPROVED_PROCESS_PATHS)

    def title(self, event):
        return f"Keylogger malware detected on [{event.get('hostIdentifier')}]"



import pypanther.helpers.panther_event_type_helpers as event_type
from pypanther.base import DataModel, DataModelMapping, LogType


def get_event_type(event):
    failed_login_events = ["credentials_failed", "mfa_failed", "modern_version_failed"]

    if event.get("category") == "success":
        return event_type.SUCCESSFUL_LOGIN

    if event.get("category") in failed_login_events:
        return event_type.FAILED_LOGIN

    return None


class StandardOnePasswordSignInAttempt(DataModel):
    id: str = "Standard.OnePassword.SignInAttempt"
    display_name: str = "1Password Signin Events"
    enabled: bool = True
    log_types: list[str] = [LogType.ONEPASSWORD_SIGNIN_ATTEMPT]
    mappings: list[DataModelMapping] = [
        DataModelMapping(name="actor_user", path="$.target_user.email"),
        DataModelMapping(name="source_ip", path="$.client.ip_address"),
        DataModelMapping(name="event_type", method=get_event_type),
    ]



from datetime import timedelta
from json import dumps

from panther_detection_helpers.caching import get_string_set, put_string_set

from pypanther import LogType, Rule, RuleMock, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get, slack_alert_context

slack_audit_logs_application_do_s_tests: list[RuleTest] = [
    RuleTest(
        name="User Session Reset - First time",
        expected_result=False,
        mocks=[
            RuleMock(object_name="get_string_set", return_value=""),
            RuleMock(object_name="put_string_set", return_value=""),
        ],
        log={
            "action": "user_session_reset_by_admin",
            "actor": {
                "type": "user",
                "user": {
                    "email": "user@example.com",
                    "id": "W012J3FEWAU",
                    "name": "primary-owner",
                    "team": "T01234N56GB",
                },
            },
            "context": {
                "ip_address": "1.2.3.4",
                "location": {
                    "domain": "test-workspace-1",
                    "id": "T01234N56GB",
                    "name": "test-workspace-1",
                    "type": "workspace",
                },
                "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
            },
        },
    ),
    RuleTest(
        name="User Session Reset - Multiple Times",
        expected_result=True,
        mocks=[
            RuleMock(object_name="get_string_set", return_value='{"time":"2021-06-08 22:24:43"}'),
            RuleMock(object_name="put_string_set", return_value=""),
        ],
        log={
            "action": "user_session_reset_by_admin",
            "actor": {
                "type": "user",
                "user": {
                    "email": "user@example.com",
                    "id": "W012J3FEWAU",
                    "name": "primary-owner",
                    "team": "T01234N56GB",
                },
            },
            "context": {
                "ip_address": "1.2.3.4",
                "location": {
                    "domain": "test-workspace-1",
                    "id": "T01234N56GB",
                    "name": "test-workspace-1",
                    "type": "workspace",
                },
                "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
            },
        },
    ),
]


@panther_managed
class SlackAuditLogsApplicationDoS(Rule):
    id = "Slack.AuditLogs.ApplicationDoS-prototype"
    display_name = "Slack Denial of Service"
    log_types = [LogType.SLACK_AUDIT_LOGS]
    tags = ["Slack", "Impact", "Endpoint Denial of Service", "Application Exhaustion Flood"]
    reports = {"MITRE ATT&CK": ["TA0040:T1499.003"]}
    default_severity = Severity.CRITICAL
    default_description = (
        "Detects when slack admin invalidates user session(s) more than once in a 24 hour period which can lead to DoS"
    )
    default_reference = "https://slack.com/intl/en-gb/help/articles/115005223763-Manage-session-duration-#pro-and-business+-subscriptions-2"
    threshold = 60
    summary_attributes = ["action", "p_any_ip_addresses", "p_any_emails"]
    tests = slack_audit_logs_application_do_s_tests
    DENIAL_OF_SERVICE_ACTIONS = [
        "bulk_session_reset_by_admin",
        "user_session_invalidated",
        "user_session_reset_by_admin",
    ]

    def rule(self, event):
        # Only evaluate actions that could be used for a DoS
        if event.get("action") not in self.DENIAL_OF_SERVICE_ACTIONS:
            return False
        # Generate a unique cache key for each user
        user_key = self.gen_key(event)
        # Retrieve prior entries from the cache, if any
        last_reset = get_string_set(user_key)
        # Store the reset info for future use
        self.store_reset_info(user_key, event)
        # If this is the first reset for the user, don't alert
        if not last_reset:
            return False
        return True

    def alert_context(self, event):
        return slack_alert_context(event)

    def gen_key(self, event):
        return f"Slack.AuditLogs.ApplicationDoS{deep_get(event, 'entity', 'user', 'name')}"

    def store_reset_info(self, key, event):
        # Map the user to the most recent reset
        put_string_set(
            key,
            [dumps({"time": event.get("p_event_time")})],
            epoch_seconds=event.event_time_epoch() + timedelta(days=1).total_seconds(),
        )



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context, deep_get
from pypanther.helpers.panther_default import aws_cloudtrail_success

awsiam_credentials_updated_tests: list[RuleTest] = [
    RuleTest(
        name="User Password Was Changed",
        expected_result=True,
        log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "IAMUser",
                "principalId": "AAAAIIIIIIU74NPJW5K76",
                "arn": "arn:aws:iam::123456789012:user/test_user",
                "accountId": "123456789012",
                "accessKeyId": "AAAAIIIIIIU74NPJW5K76",
                "userName": "test_user",
                "sessionContext": {"attributes": {"mfaAuthenticated": "true", "creationDate": "2019-12-31T01:50:17Z"}},
                "invokedBy": "signin.amazonaws.com",
            },
            "eventTime": "2019-12-31T01:50:46Z",
            "eventSource": "iam.amazonaws.com",
            "eventName": "ChangePassword",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "64.25.27.224",
            "userAgent": "signin.amazonaws.com",
            "requestParameters": None,
            "responseElements": None,
            "requestID": "a431f05e-67e1-11ea-bc55-0242ac130003",
            "eventID": "a431f05e-67e1-11ea-bc55-0242ac130003",
            "readOnly": False,
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
        },
    ),
    RuleTest(
        name="MFA Device Was Created",
        expected_result=False,
        log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "IAMUser",
                "principalId": "AAAAIIIIIIU74NPJW5K76",
                "arn": "arn:aws:iam::123456789012:user/test_user",
                "accountId": "123456789012",
                "accessKeyId": "AAAAIIIIIIU74NPJW5K76",
                "userName": "test_user",
                "sessionContext": {"attributes": {"mfaAuthenticated": "true", "creationDate": "2019-12-31T01:50:17Z"}},
                "invokedBy": "signin.amazonaws.com",
            },
            "eventTime": "2019-12-31T01:50:46Z",
            "eventSource": "iam.amazonaws.com",
            "eventName": "CreateVirtualMFADevice",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "64.25.27.224",
            "userAgent": "signin.amazonaws.com",
            "requestParameters": None,
            "responseElements": None,
            "requestID": "a431f05e-67e1-11ea-bc55-0242ac130003",
            "eventID": "a431f05e-67e1-11ea-bc55-0242ac130003",
            "readOnly": False,
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
        },
    ),
    RuleTest(
        name="User Password Change Error",
        expected_result=False,
        log={
            "eventVersion": "1.05",
            "errorCode": "PasswordPolicyViolation",
            "userIdentity": {
                "type": "IAMUser",
                "principalId": "AAAAIIIIIIU74NPJW5K76",
                "arn": "arn:aws:iam::123456789012:user/test_user",
                "accountId": "123456789012",
                "accessKeyId": "AAAAIIIIIIU74NPJW5K76",
                "userName": "test_user",
                "sessionContext": {"attributes": {"mfaAuthenticated": "true", "creationDate": "2019-12-31T01:50:17Z"}},
                "invokedBy": "signin.amazonaws.com",
            },
            "eventTime": "2019-12-31T01:50:46Z",
            "eventSource": "iam.amazonaws.com",
            "eventName": "ChangePassword",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "64.25.27.224",
            "userAgent": "signin.amazonaws.com",
            "requestParameters": None,
            "responseElements": None,
            "requestID": "a431f05e-67e1-11ea-bc55-0242ac130003",
            "eventID": "a431f05e-67e1-11ea-bc55-0242ac130003",
            "readOnly": False,
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
        },
    ),
]


@panther_managed
class AWSIAMCredentialsUpdated(Rule):
    id = "AWS.IAM.CredentialsUpdated-prototype"
    display_name = "New IAM Credentials Updated"
    log_types = [LogType.AWS_CLOUDTRAIL]
    reports = {"MITRE ATT&CK": ["TA0003:T1098"]}
    tags = ["AWS", "Identity & Access Management", "Persistence:Account Manipulation"]
    default_severity = Severity.INFO
    default_description = "A console password, access key, or user has been created."
    default_runbook = "This rule is purely informational, there is no action needed."
    default_reference = "https://docs.aws.amazon.com/IAM/latest/UserGuide/list_identityandaccessmanagement.html"
    summary_attributes = ["eventName", "userAgent", "sourceIpAddress", "recipientAccountId", "p_any_aws_arns"]
    tests = awsiam_credentials_updated_tests
    UPDATE_EVENTS = {"ChangePassword", "CreateAccessKey", "CreateLoginProfile", "CreateUser"}

    def rule(self, event):
        return event.get("eventName") in self.UPDATE_EVENTS and aws_cloudtrail_success(event)

    def dedup(self, event):
        return deep_get(event, "userIdentity", "userName", default="<UNKNOWN_USER>")

    def title(self, event):
        return f"{deep_get(event, 'userIdentity', 'type')} [{deep_get(event, 'userIdentity', 'arn')}] has updated their IAM credentials"

    def alert_context(self, event):
        return aws_rule_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context, deep_get

aws_suspicious_saml_activity_tests: list[RuleTest] = [
    RuleTest(
        name="CreateSAMLProvider",
        expected_result=True,
        log={
            "awsRegion": "us-east-1",
            "eventID": "EID12345",
            "eventName": "CreateSAMLProvider",
            "eventSource": "iam.amazonaws.com",
            "eventTime": "2021-10-14 21:25:20",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "recipientAccountId": "0123456789",
            "requestID": "ABC1234",
            "sourceIPAddress": "1.2.3.4",
            "userAgent": "cloudformation.amazonaws.com",
            "userIdentity": {
                "accessKeyId": "ABCDEFGHIJK",
                "accountId": "0123456789",
                "arn": "arn:aws:sts::0123456789:assumed-role/role/account",
                "invokedBy": "cloudformation.amazonaws.com",
                "principalId": "0123456789:AWSCloudFormation",
                "sessionContext": {
                    "attributes": {"creationDate": "2021-10-14T21:25:20Z", "mfaAuthenticated": "false"},
                    "sessionIssuer": {
                        "accountId": "0123456789",
                        "arn": "arn:aws:iam::0123456789:role/ServiceRole",
                        "principalId": "ABCDEFGI0123",
                        "type": "Role",
                        "userName": "ServiceRole",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="DeleteSAMLProvider",
        expected_result=True,
        log={
            "awsRegion": "us-east-1",
            "eventID": "EID12345",
            "eventName": "DeleteSAMLProvider",
            "eventSource": "iam.amazonaws.com",
            "eventTime": "2021-10-14 21:25:20",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "recipientAccountId": "0123456789",
            "requestID": "ABC1234",
            "sourceIPAddress": "1.2.3.4",
            "userAgent": "cloudformation.amazonaws.com",
            "userIdentity": {
                "accessKeyId": "ABCDEFGHIJK",
                "accountId": "0123456789",
                "arn": "arn:aws:sts::0123456789:assumed-role/role/account",
                "invokedBy": "cloudformation.amazonaws.com",
                "principalId": "0123456789:AWSCloudFormation",
                "sessionContext": {
                    "attributes": {"creationDate": "2021-10-14T21:25:20Z", "mfaAuthenticated": "false"},
                    "sessionIssuer": {
                        "accountId": "0123456789",
                        "arn": "arn:aws:iam::0123456789:role/ServiceRole",
                        "principalId": "ABCDEFGI0123",
                        "type": "Role",
                        "userName": "ServiceRole",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="Non Target Event",
        expected_result=False,
        log={
            "awsRegion": "us-east-1",
            "eventID": "EID12345",
            "eventName": "ListAccessKeys",
            "eventSource": "iam.amazonaws.com",
            "eventTime": "2021-10-13 18:35:08",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "readOnly": True,
            "recipientAccountId": "0123456789",
            "requestID": "requestID12345",
            "sourceIPAddress": "1.2.3.4",
            "userAgent": "console.amazonaws.com",
            "userIdentity": {
                "accessKeyId": "ABCDEFGHIJKLMNOP",
                "accountId": "0123456789",
                "arn": "arn:aws:iam::0123456789:user/bob",
                "principalId": "ABCDEF012345",
                "sessionContext": {
                    "attributes": {"creationDate": "2021-10-13T18:35:02Z", "mfaAuthenticated": "true"},
                    "sessionIssuer": {},
                    "webIdFederationData": {},
                },
                "type": "IAMUser",
                "userName": "bob",
            },
        },
    ),
    RuleTest(
        name="UpdateSAMLProvider",
        expected_result=True,
        log={
            "awsRegion": "us-east-1",
            "eventID": "EID12345",
            "eventName": "UpdateSAMLProvider",
            "eventSource": "iam.amazonaws.com",
            "eventTime": "2021-10-14 21:25:20",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "recipientAccountId": "0123456789",
            "requestID": "ABC1234",
            "sourceIPAddress": "1.2.3.4",
            "userAgent": "cloudformation.amazonaws.com",
            "userIdentity": {
                "accessKeyId": "ABCDEFGHIJK",
                "accountId": "0123456789",
                "arn": "arn:aws:sts::0123456789:assumed-role/role/account",
                "invokedBy": "cloudformation.amazonaws.com",
                "principalId": "0123456789:AWSCloudFormation",
                "sessionContext": {
                    "attributes": {"creationDate": "2021-10-14T21:25:20Z", "mfaAuthenticated": "false"},
                    "sessionIssuer": {
                        "accountId": "0123456789",
                        "arn": "arn:aws:iam::0123456789:role/ServiceRole",
                        "principalId": "ABCDEFGI0123",
                        "type": "Role",
                        "userName": "ServiceRole",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="Activity from AWSSSO Service Managed Role",
        expected_result=False,
        log={
            "awsRegion": "us-east-1",
            "eventCategory": "Management",
            "eventName": "CreateSAMLProvider",
            "eventSource": "iam.amazonaws.com",
            "eventTime": "2022-12-12 21:46:17.000000000",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "p_alert_context": {
                "awsRegion": "us-east-1",
                "eventName": "CreateSAMLProvider",
                "eventSource": "iam.amazonaws.com",
                "recipientAccountId": "123412341234",
                "sourceIPAddress": "sso.amazonaws.com",
                "userAgent": "sso.amazonaws.com",
                "userIdentity": {
                    "accessKeyId": "ASIAXXXXNLMHSP3MFXX",
                    "accountId": "123412341234",
                    "arn": "arn:aws:sts::123412341234:assumed-role/AWSServiceRoleForSSO/AWS-SSO",
                    "invokedBy": "sso.amazonaws.com",
                    "principalId": "AROAT7BCMNLMONMOFFFFF:AWS-SSO",
                    "sessionContext": {
                        "attributes": {"creationDate": "2022-12-12T21:46:16Z", "mfaAuthenticated": "false"},
                        "sessionIssuer": {
                            "accountId": "123412341234",
                            "arn": "arn:aws:iam::123412341234:role/aws-service-role/sso.amazonaws.com/AWSServiceRoleForSSO",
                            "principalId": "AROAT7BCMNLMONMOFFFFF",
                            "type": "Role",
                            "userName": "AWSServiceRoleForSSO",
                        },
                        "webIdFederationData": {},
                    },
                    "type": "AssumedRole",
                },
            },
            "p_alert_creation_time": "2022-12-12 21:51:37.115853000",
            "p_alert_update_time": "2022-12-12 21:51:37.115853000",
            "p_any_aws_account_ids": ["123412341234"],
            "p_any_aws_arns": [
                "arn:aws:iam::123412341234:role/aws-service-role/sso.amazonaws.com/AWSServiceRoleForSSO",
                "arn:aws:iam::123412341234:saml-provider/AWSSSO_abdf34fd171b4a7e_DO_NOT_DELETE",
                "arn:aws:sts::123412341234:assumed-role/AWSServiceRoleForSSO/AWS-SSO",
            ],
            "p_any_domain_names": ["sso.amazonaws.com"],
            "p_any_trace_ids": ["ASIAXXXXNLMHSP3MFXX"],
            "p_any_usernames": ["AWSServiceRoleForSSO"],
            "p_event_time": "2022-12-12 21:46:17.000000000",
            "p_log_type": "AWS.CloudTrail",
            "p_parse_time": "2022-12-12 21:49:13.694384486",
            "p_rule_id": "AWS.Suspicious.SAML.Activity",
            "p_source_label": "YourOrg - Cloudtrail - Label",
            "readOnly": False,
            "recipientAccountId": "123412341234",
            "requestID": "cb89df1f-6019-427f-9a69-00b8b904ce0d",
            "requestParameters": {
                "name": "AWSSSO_abdf34fd171b4a7e_DO_NOT_DELETE",
                "sAMLMetadataDocument": '<?xml version="1.0" encoding="UTF-8"?></xml>',
            },
            "responseElements": {
                "sAMLProviderArn": "arn:aws:iam::123412341234:saml-provider/AWSSSO_abdf34fd171b4a7e_DO_NOT_DELETE",
            },
            "sourceIPAddress": "sso.amazonaws.com",
            "userAgent": "sso.amazonaws.com",
            "userIdentity": {
                "accessKeyId": "ASIAXXXXNLMHSP3MFXX",
                "accountId": "123412341234",
                "arn": "arn:aws:sts::123412341234:assumed-role/AWSServiceRoleForSSO/AWS-SSO",
                "invokedBy": "sso.amazonaws.com",
                "principalId": "AROAT7BCMNLMONMOFFFFF:AWS-SSO",
                "sessionContext": {
                    "attributes": {"creationDate": "2022-12-12T21:46:16Z", "mfaAuthenticated": "false"},
                    "sessionIssuer": {
                        "accountId": "123412341234",
                        "arn": "arn:aws:iam::123412341234:role/aws-service-role/sso.amazonaws.com/AWSServiceRoleForSSO",
                        "principalId": "AROAT7BCMNLMONMOFFFFF",
                        "type": "Role",
                        "userName": "AWSServiceRoleForSSO",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
]


@panther_managed
class AWSSuspiciousSAMLActivity(Rule):
    default_description = (
        "Identifies when SAML activity has occurred in AWS. An adversary could gain backdoor access via SAML."
    )
    display_name = "AWS SAML Activity"
    default_reference = (
        "https://docs.aws.amazon.com/cognito/latest/developerguide/cognito-user-pools-managing-saml-idp-console.html"
    )
    default_severity = Severity.MEDIUM
    log_types = [LogType.AWS_CLOUDTRAIL]
    id = "AWS.Suspicious.SAML.Activity-prototype"
    tests = aws_suspicious_saml_activity_tests
    SAML_ACTIONS = ["UpdateSAMLProvider", "CreateSAMLProvider", "DeleteSAMLProvider"]

    def rule(self, event):
        # Allow AWSSSO to manage
        if deep_get(event, "userIdentity", "arn", default="").endswith(":assumed-role/AWSServiceRoleForSSO/AWS-SSO"):
            return False
        return event.get("eventSource") == "iam.amazonaws.com" and event.get("eventName") in self.SAML_ACTIONS

    def title(self, event):
        return f"[{deep_get(event, 'userIdentity', 'arn')}] performed [{event.get('eventName')}] in account [{event.get('recipientAccountId')}]"

    def alert_context(self, event):
        return aws_rule_context(event)



"""Default Configuration values for Panther"""

"""
Here, default values for `panther_config.config` are defined
"""

# A list of public DNS domain names that fall under the administrative domain of
# the Panther installation
ORGANIZATION_DOMAINS = ["example.com"]

DROPBOX_ALLOWED_SHARE_DOMAINS = ORGANIZATION_DOMAINS
DROPBOX_TRUSTED_OWNERSHIP_DOMAINS = ORGANIZATION_DOMAINS
GSUITE_TRUSTED_FORWARDING_DESTINATION_DOMAINS = ORGANIZATION_DOMAINS
GSUITE_TRUSTED_OWNERSHIP_DOMAINS = ORGANIZATION_DOMAINS
MS_EXCHANGE_ALLOWED_FORWARDING_DESTINATION_DOMAINS = ORGANIZATION_DOMAINS
MS_EXCHANGE_ALLOWED_FORWARDING_DESTINATION_EMAILS = ["postmaster@" + ORGANIZATION_DOMAINS[0]]
TELEPORT_ORGANIZATION_DOMAINS = ORGANIZATION_DOMAINS

DMZ_NETWORKS = [
    # ip_network("10.1.0.0/24"),
]

DMZ_TAGS = set(
    [
        ("environment", "dmz"),
    ],
)

PCI_NETWORKS = [
    # ip_network("10.0.0.0/24"),
]



from collections.abc import Mapping

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context, deep_get
from pypanther.helpers.panther_default import aws_cloudtrail_success

aws_cloud_trail_snapshot_made_public_tests: list[RuleTest] = [
    RuleTest(
        name="Snapshot Made Publicly Accessible",
        expected_result=True,
        log={
            "awsRegion": "us-west-2",
            "eventID": "1111",
            "eventName": "ModifySnapshotAttribute",
            "eventSource": "ec2.amazonaws.com",
            "eventTime": "2019-01-01T00:00:00Z",
            "eventType": "AwsApiCall",
            "eventVersion": "1.05",
            "recipientAccountId": "123456789012",
            "requestID": "1111",
            "requestParameters": {
                "attributeType": "CREATE_VOLUME_PERMISSION",
                "createVolumePermission": {"add": {"items": [{"group": "all"}]}},
                "snapshotId": "snap-1111",
            },
            "responseElements": {"_return": True, "requestId": "1111"},
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla/2.0 (compatible; NEWT ActiveX; Win32)",
            "userIdentity": {
                "accessKeyId": "1111",
                "accountId": "123456789012",
                "arn": "arn:aws:sts::123456789012:assumed-role/example-role/example-user",
                "principalId": "1111",
                "sessionContext": {
                    "attributes": {"creationDate": "2019-01-01T00:00:00Z", "mfaAuthenticated": "true"},
                    "sessionIssuer": {
                        "accountId": "123456789012",
                        "arn": "arn:aws:iam::123456789012:role/example-role",
                        "principalId": "1111",
                        "type": "Role",
                        "userName": "example-role",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="Snapshot Not Made Publicly Accessible",
        expected_result=False,
        log={
            "awsRegion": "us-west-2",
            "eventID": "1111",
            "eventName": "ModifySnapshotAttribute",
            "eventSource": "ec2.amazonaws.com",
            "eventTime": "2019-01-01T00:00:00Z",
            "eventType": "AwsApiCall",
            "eventVersion": "1.05",
            "recipientAccountId": "123456789012",
            "requestID": "1111",
            "requestParameters": {
                "attributeType": "CREATE_VOLUME_PERMISSION",
                "createVolumePermission": {"add": {"items": [{"group": "none"}]}},
                "snapshotId": "snap-1111",
            },
            "responseElements": {"_return": True, "requestId": "1111"},
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla/2.0 (compatible; NEWT ActiveX; Win32)",
            "userIdentity": {
                "accessKeyId": "1111",
                "accountId": "123456789012",
                "arn": "arn:aws:sts::123456789012:assumed-role/example-role/example-user",
                "principalId": "1111",
                "sessionContext": {
                    "attributes": {"creationDate": "2019-01-01T00:00:00Z", "mfaAuthenticated": "true"},
                    "sessionIssuer": {
                        "accountId": "123456789012",
                        "arn": "arn:aws:iam::123456789012:role/example-role",
                        "principalId": "1111",
                        "type": "Role",
                        "userName": "example-role",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="Error Making Snapshot Publicly Accessible",
        expected_result=False,
        log={
            "awsRegion": "us-west-2",
            "errorCode": "ValidationError",
            "eventID": "1111",
            "eventName": "ModifySnapshotAttribute",
            "eventSource": "ec2.amazonaws.com",
            "eventTime": "2019-01-01T00:00:00Z",
            "eventType": "AwsApiCall",
            "eventVersion": "1.05",
            "recipientAccountId": "123456789012",
            "requestID": "1111",
            "requestParameters": {
                "attributeType": "CREATE_VOLUME_PERMISSION",
                "createVolumePermission": {"add": {"items": [{"group": "all"}]}},
                "snapshotId": "snap-1111",
            },
            "responseElements": {"_return": True, "requestId": "1111"},
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla/2.0 (compatible; NEWT ActiveX; Win32)",
            "userIdentity": {
                "accessKeyId": "1111",
                "accountId": "123456789012",
                "arn": "arn:aws:sts::123456789012:assumed-role/example-role/example-user",
                "principalId": "1111",
                "sessionContext": {
                    "attributes": {"creationDate": "2019-01-01T00:00:00Z", "mfaAuthenticated": "true"},
                    "sessionIssuer": {
                        "accountId": "123456789012",
                        "arn": "arn:aws:iam::123456789012:role/example-role",
                        "principalId": "1111",
                        "type": "Role",
                        "userName": "example-role",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
]


@panther_managed
class AWSCloudTrailSnapshotMadePublic(Rule):
    id = "AWS.CloudTrail.SnapshotMadePublic-prototype"
    display_name = "AWS Snapshot Made Public"
    log_types = [LogType.AWS_CLOUDTRAIL]
    tags = ["AWS", "Exfiltration:Transfer Data to Cloud Account"]
    reports = {"MITRE ATT&CK": ["TA0010:T1537"]}
    default_severity = Severity.MEDIUM
    default_description = "An AWS storage snapshot was made public."
    default_runbook = "Adjust the snapshot configuration so that it is no longer public."
    default_reference = "https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ebs-modifying-snapshot-permissions.html"
    summary_attributes = ["userAgent", "sourceIpAddress", "recipientAccountId", "p_any_aws_arns"]
    tests = aws_cloud_trail_snapshot_made_public_tests

    def rule(self, event):
        if not aws_cloudtrail_success(event):
            return False
        # EC2 Volume snapshot made public
        if event.get("eventName") == "ModifySnapshotAttribute":
            parameters = event.get("requestParameters", {})
            if parameters.get("attributeType") != "CREATE_VOLUME_PERMISSION":
                return False
            items = deep_get(parameters, "createVolumePermission", "add", "items", default=[])
            for item in items:
                if not isinstance(item, (Mapping, dict)):
                    continue
                if item.get("userId") or item.get("group") == "all":
                    return True
            return False
        return False

    def alert_context(self, event):
        return aws_rule_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

zendesk_new_api_token_tests: list[RuleTest] = [
    RuleTest(
        name="Zendesk - API Token Updated",
        expected_result=False,
        log={
            "url": "https://myzendek.zendesk.com/api/v2/audit_logs/111222333444.json",
            "id": 123456789123,
            "action_label": "Updated",
            "actor_id": 123,
            "actor_name": "John Doe",
            "source_id": 123,
            "source_type": "api_token",
            "source_label": "API token: a new description",
            "action": "update",
            "change_description": "description changed from not set to a new description",
            "ip_address": "127.0.0.1",
            "created_at": "2021-05-28T18:39:50Z",
            "p_log_type": "Zendesk.Audit",
        },
    ),
    RuleTest(
        name="Zendesk - API Token Created",
        expected_result=True,
        log={
            "url": "https://myzendek.zendesk.com/api/v2/audit_logs/111222333444.json",
            "id": 123456789123,
            "action_label": "Created",
            "actor_id": 123,
            "actor_name": "John Doe",
            "source_id": 123,
            "source_type": "api_token",
            "source_label": "API token",
            "action": "create",
            "change_description": "",
            "ip_address": "127.0.0.1",
            "created_at": "2021-05-28T18:39:50Z",
            "p_log_type": "Zendesk.Audit",
        },
    ),
]


@panther_managed
class ZendeskNewAPIToken(Rule):
    id = "Zendesk.NewAPIToken-prototype"
    display_name = "Zendesk API Token Created"
    log_types = [LogType.ZENDESK_AUDIT]
    default_severity = Severity.HIGH
    tags = ["Zendesk", "Credential Access:Steal Application Access Token"]
    reports = {"MITRE ATT&CK": ["TA0006:T1528"]}
    default_description = "A user created a new API token to be used with Zendesk."
    default_runbook = "Validate the api token was created for valid use case, otherwise delete the token immediately."
    default_reference = "https://support.zendesk.com/hc/en-us/articles/4408889192858-Managing-access-to-the-Zendesk-API#topic_bsw_lfg_mmb:~:text=enable%20token%20access.-,Generating%20API%20tokens,-To%20generate%20an"
    summary_attributes = ["p_any_ip_addresses"]
    tests = zendesk_new_api_token_tests
    API_TOKEN_ACTIONS = {"create", "destroy"}

    def rule(self, event):
        return event.get("source_type") == "api_token" and event.get("action", "") in self.API_TOKEN_ACTIONS

    def title(self, event):
        action = event.get("action", "<UNKNOWN_ACTION>")
        return f"[{event.get('p_log_type')}]: User [{event.udm('actor_user')}] {action} an api token"

    def severity(self, event):
        if event.get("action", "") == "destroy":
            return "INFO"
        return "HIGH"



import re

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_auth0_helpers import auth0_alert_context, is_auth0_config_event
from pypanther.helpers.panther_base_helpers import deep_get

auth0_user_invitation_created_tests: list[RuleTest] = [
    RuleTest(
        name="Test-org",
        expected_result=True,
        log={
            "data": {
                "_id": "90020230616045255729813000000000000001223372038324184656",
                "client_id": "6xNLmMWZMYvMO3ZjQoN8siUWAbg3pnpA",
                "client_name": "",
                "date": "2023-06-16T04:52:50.663Z",
                "description": "Create invitations to organization",
                "details": {
                    "request": {
                        "auth": {
                            "credentials": {"jti": "81a67a5a3b2c4fb5cc2fcf38349456dd"},
                            "strategy": "jwt",
                            "user": {
                                "email": "bob@panther.com",
                                "name": "Bob",
                                "user_id": "google-oauth2|115547599209686809398",
                            },
                        },
                        "body": {
                            "client_id": "KwJItGFu62zryEc4c8t5BQuwB1qdeDFa",
                            "invitee": {"email": "larry@example.com"},
                            "inviter": {"name": "Larry Jones"},
                        },
                        "channel": "https://manage.auth0.com/",
                        "ip": "123.123.123.123",
                        "method": "post",
                        "path": "/api/v2/organizations/org_tFmw9RlOUjoSkOf1/invitations",
                        "query": {},
                        "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/114.0.0.0 Safari/537.36",
                    },
                    "response": {
                        "body": {
                            "client_id": "KwJItGFu62zryEc4c8t5BQuwB1qdeDFa",
                            "id": "uinv_DeLuEdgf3hjxRd0z",
                            "invitee": {"email": "frank@example.com"},
                            "inviter": {"name": "Bob Jones"},
                        },
                        "statusCode": 201,
                    },
                },
                "id": "90020230616045255729813000000000000001223372038324184656",
                "ip": "123.123.123.123",
                "isMobile": False,
                "log_id": "90020230616045255729813000000000000001223372038324184656",
                "type": "sapi",
                "user_agent": "Chrome 114.0.0 / Mac OS X 10.15.7",
                "user_id": "google-oauth2|115547599209686809398",
            },
            "p_any_ip_addresses": ["12.12.12.12"],
            "p_any_usernames": ["auth0|6459776e974703f3a65dc258"],
            "p_event_time": "2023-05-15 16:13:53.609",
            "p_log_type": "Auth0.Events",
            "p_parse_time": "2023-05-15 16:15:28.555",
            "p_row_id": "e20ac28001d19ac6df97b99618d4a207",
            "p_schema_version": 0,
            "p_source_id": "b9031579-b2c5-45c2-b15c-632b995a4e36",
            "p_source_label": "Auth0 Org Label",
        },
    ),
    RuleTest(
        name="Test-org-regex-fail",
        expected_result=False,
        log={
            "data": {
                "_id": "90020230616045255729813000000000000001223372038324184656",
                "client_id": "6xNLmMWZMYvMO3ZjQoN8siUWAbg3pnpA",
                "client_name": "",
                "date": "2023-06-16T04:52:50.663Z",
                "description": "Create invitations to organization",
                "details": {
                    "request": {
                        "auth": {
                            "credentials": {"jti": "81a67a5a3b2c4fb5cc2fcf38349456dd"},
                            "strategy": "jwt",
                            "user": {
                                "email": "bob@panther.com",
                                "name": "Bob",
                                "user_id": "google-oauth2|115547599209686809398",
                            },
                        },
                        "body": {
                            "client_id": "KwJItGFu62zryEc4c8t5BQuwB1qdeDFa",
                            "invitee": {"email": "larry@example.com"},
                            "inviter": {"name": "Larry Jones"},
                        },
                        "channel": "https://manage.auth0.com/",
                        "ip": "123.123.123.123",
                        "method": "post",
                        "path": "/api/v2/organizations/more/org_tFmw9RlOUjoSkOf1/invitations",
                        "query": {},
                        "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/114.0.0.0 Safari/537.36",
                    },
                    "response": {
                        "body": {
                            "client_id": "KwJItGFu62zryEc4c8t5BQuwB1qdeDFa",
                            "id": "uinv_DeLuEdgf3hjxRd0z",
                            "invitee": {"email": "frank@example.com"},
                            "inviter": {"name": "Bob Jones"},
                        },
                        "statusCode": 201,
                    },
                },
                "id": "90020230616045255729813000000000000001223372038324184656",
                "ip": "35.167.74.121",
                "isMobile": False,
                "log_id": "90020230616045255729813000000000000001223372038324184656",
                "type": "sapi",
                "user_agent": "Chrome 114.0.0 / Mac OS X 10.15.7",
                "user_id": "google-oauth2|115547599209686809398",
            },
            "p_any_ip_addresses": ["12.12.12.12"],
            "p_any_usernames": ["auth0|6459776e974703f3a65dc258"],
            "p_event_time": "2023-05-15 16:13:53.609",
            "p_log_type": "Auth0.Events",
            "p_parse_time": "2023-05-15 16:15:28.555",
            "p_row_id": "e20ac28001d19ac6df97b99618d4a207",
            "p_schema_version": 0,
            "p_source_id": "b9031579-b2c5-45c2-b15c-632b995a4e36",
            "p_source_label": "Auth0 Tenant Label",
        },
    ),
    RuleTest(
        name="Test-other-endpoint",
        expected_result=False,
        log={
            "data": {
                "client_id": "1HXWWGKk1Zj3JF8GvMrnCSirccDs4qvr",
                "client_name": "",
                "date": "2023-05-15 16:13:53.609000000",
                "description": "Create tenant invitations for a given client",
                "details": {
                    "request": {
                        "auth": {
                            "credentials": {
                                "jti": "dc1843dbe925a1ed2e707452c2123913",
                                "scopes": ["create:actions", "create:actions_log_sessions"],
                            },
                            "strategy": "jwt",
                            "user": {
                                "email": "homer.simpson@yourcompany.io",
                                "name": "homer.simpson@yourcompany.io",
                                "user_id": "auth0|6459776e974703f3a65dc258",
                            },
                        },
                        "body": {"owners": ["marge.simpson@yourcompany.io"], "roles": ["owner"]},
                        "channel": "https://manage.auth0.com/",
                        "ip": "12.12.12.12",
                        "method": "post",
                        "path": "/api/v2/some-other-endpoint",
                        "query": {},
                        "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
                    },
                    "response": {
                        "body": [
                            {
                                "email": "marge.simpson@yourcompany.io",
                                "expires_at": "2023-05-18T16:13:53.600Z",
                                "invitation_id": "inv_TEyzbreI336AHrfU",
                            },
                        ],
                        "statusCode": 201,
                    },
                },
                "ip": "12.12.12.12",
                "log_id": "90020230515161358744602000000000000001223372037485092159",
                "type": "sapi",
                "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
                "user_id": "auth0|6459776e974703f3a65dc258",
            },
            "log_id": "90020230515161358744602000000000000001223372037485092159",
            "p_any_ip_addresses": ["12.12.12.12"],
            "p_any_usernames": ["auth0|6459776e974703f3a65dc258"],
            "p_event_time": "2023-05-15 16:13:53.609",
            "p_log_type": "Auth0.Events",
            "p_parse_time": "2023-05-15 16:15:28.555",
            "p_row_id": "e20ac28001d19ac6df97b99618d4a207",
            "p_schema_version": 0,
            "p_source_id": "b9031579-b2c5-45c2-b15c-632b995a4e36",
            "p_source_label": "Org Auth0 Tenant Label",
        },
    ),
    RuleTest(
        name="Test-no-invitee",
        expected_result=True,
        log={
            "data": {
                "client_id": "1HXWWGKk1Zj3JF8GvMrnCSirccDs4qvr",
                "client_name": "",
                "date": "2023-05-15 16:13:53.609000000",
                "description": "Create tenant invitations for a given client",
                "details": {
                    "request": {
                        "auth": {
                            "credentials": {
                                "jti": "dc1843dbe925a1ed2e707452c2123913",
                                "scopes": ["create:actions", "create:actions_log_sessions"],
                            },
                            "strategy": "jwt",
                            "user": {
                                "email": "homer.simpson@yourcompany.io",
                                "name": "homer.simpson@yourcompany.io",
                                "user_id": "auth0|6459776e974703f3a65dc258",
                            },
                        },
                        "body": None,
                        "channel": "https://manage.auth0.com/",
                        "ip": "12.12.12.12",
                        "method": "post",
                        "path": "/api/v2/tenants/invitations",
                        "query": {},
                        "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
                    },
                    "response": {
                        "body": [
                            {
                                "email": "marge.simpson@yourcompany.io",
                                "expires_at": "2023-05-18T16:13:53.600Z",
                                "invitation_id": "inv_TEyzbreI336AHrfU",
                            },
                        ],
                        "statusCode": 201,
                    },
                },
                "ip": "12.12.12.12",
                "log_id": "90020230515161358744602000000000000001223372037485092159",
                "type": "sapi",
                "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
                "user_id": "auth0|6459776e974703f3a65dc258",
            },
            "log_id": "90020230515161358744602000000000000001223372037485092159",
            "p_any_ip_addresses": ["12.12.12.12"],
            "p_any_usernames": ["auth0|6459776e974703f3a65dc258"],
            "p_event_time": "2023-05-15 16:13:53.609",
            "p_log_type": "Auth0.Events",
            "p_parse_time": "2023-05-15 16:15:28.555",
            "p_row_id": "e20ac28001d19ac6df97b99618d4a207",
            "p_schema_version": 0,
            "p_source_id": "b9031579-b2c5-45c2-b15c-632b995a4e36",
            "p_source_label": "Auth0 Tenant Label",
        },
    ),
    RuleTest(
        name="Test-tenant",
        expected_result=True,
        log={
            "data": {
                "client_id": "1HXWWGKk1Zj3JF8GvMrnCSirccDs4qvr",
                "client_name": "",
                "date": "2023-05-15 16:13:53.609000000",
                "description": "Create tenant invitations for a given client",
                "details": {
                    "request": {
                        "auth": {
                            "credentials": {
                                "jti": "dc1843dbe925a1ed2e707452c2123913",
                                "scopes": ["create:actions", "create:actions_log_sessions"],
                            },
                            "strategy": "jwt",
                            "user": {
                                "email": "homer.simpson@yourcompany.io",
                                "name": "homer.simpson@yourcompany.io",
                                "user_id": "auth0|6459776e974703f3a65dc258",
                            },
                        },
                        "body": {"owners": ["marge.simpson@yourcompany.io"], "roles": ["owner"]},
                        "channel": "https://manage.auth0.com/",
                        "ip": "12.12.12.12",
                        "method": "post",
                        "path": "/api/v2/tenants/invitations",
                        "query": {},
                        "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
                    },
                    "response": {
                        "body": [
                            {
                                "email": "marge.simpson@yourcompany.io",
                                "expires_at": "2023-05-18T16:13:53.600Z",
                                "invitation_id": "inv_TEyzbreI336AHrfU",
                            },
                        ],
                        "statusCode": 201,
                    },
                },
                "ip": "12.12.12.12",
                "log_id": "90020230515161358744602000000000000001223372037485092159",
                "type": "sapi",
                "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
                "user_id": "auth0|6459776e974703f3a65dc258",
            },
            "log_id": "90020230515161358744602000000000000001223372037485092159",
            "p_any_ip_addresses": ["12.12.12.12"],
            "p_any_usernames": ["auth0|6459776e974703f3a65dc258"],
            "p_event_time": "2023-05-15 16:13:53.609",
            "p_log_type": "Auth0.Events",
            "p_parse_time": "2023-05-15 16:15:28.555",
            "p_row_id": "e20ac28001d19ac6df97b99618d4a207",
            "p_schema_version": 0,
            "p_source_id": "b9031579-b2c5-45c2-b15c-632b995a4e36",
            "p_source_label": "Auth0 Tenant Label",
        },
    ),
]


@panther_managed
class Auth0UserInvitationCreated(Rule):
    display_name = "Auth0 User Invitation Created"
    default_reference = "https://auth0.com/docs/manage-users/organizations/configure-organizations/invite-members"
    default_severity = Severity.INFO
    log_types = [LogType.AUTH0_EVENTS]
    id = "Auth0.User.Invitation.Created-prototype"
    tests = auth0_user_invitation_created_tests
    org_re = re.compile("^/api/v2/organizations/[^/\\s]+/invitations$")

    def rule(self, event):
        if not any([True, is_auth0_config_event(event)]):
            return False
        return self.invitation_type(event) is not None

    def title(self, event):
        inv_type = self.invitation_type(event)
        if inv_type == "tenant":
            try:
                invitee = deep_get(event, "data", "details", "request", "body", "owners", default=[])[0]
            except IndexError:
                invitee = "<NO_INVITEE>"
        elif inv_type == "organization":
            invitee = deep_get(event, "data", "details", "request", "body", "invitee", "email")
        else:
            invitee = "<NO_INVITEE>"
        inviter = deep_get(event, "data", "details", "request", "auth", "user", "email", default="<NO_INVITER>")
        source = deep_get(event, "p_source_label", default="<NO_PSOURCE>")
        return f"Auth0 User [{inviter}] invited [{invitee}] to {inv_type} [{source}]]"

    def invitation_type(self, event):
        path = deep_get(event, "data", "details", "request", "path", default="")
        if path == "/api/v2/tenants/invitations":
            return "tenant"
        if self.org_re.match(path):
            return "organization"
        return None

    def alert_context(self, event):
        return auth0_alert_context(event)



import re

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_config import config

teleport_saml_login_without_company_domain_tests: list[RuleTest] = [
    RuleTest(
        name="A user authenticated with SAML, but from a known company domain",
        expected_result=False,
        log={
            "attributes": {"firstName": [""], "groups": ["employees"]},
            "cluster_name": "teleport.example.com",
            "code": "T1001I",
            "ei": 0,
            "event": "user.login",
            "method": "saml",
            "success": True,
            "time": "2023-09-18 00:00:00",
            "uid": "88888888-4444-4444-4444-222222222222",
            "user": "jane.doe@example.com",
        },
    ),
    RuleTest(
        name="A user authenticated with SAML, but not from a company domain",
        expected_result=True,
        log={
            "cluster_name": "teleport.example.com",
            "code": "T1001I",
            "ei": 0,
            "event": "user.login",
            "method": "saml",
            "success": True,
            "time": "2023-09-18 00:00:00",
            "uid": "88888888-4444-4444-4444-222222222222",
            "user": "wtf.how@omghax.gravitational.io",
        },
    ),
]


@panther_managed
class TeleportSAMLLoginWithoutCompanyDomain(Rule):
    id = "Teleport.SAMLLoginWithoutCompanyDomain-prototype"
    display_name = "A user authenticated with SAML, but from an unknown company domain"
    log_types = [LogType.GRAVITATIONAL_TELEPORT_AUDIT]
    tags = ["Teleport"]
    default_severity = Severity.HIGH
    default_description = "A user authenticated with SAML, but from an unknown company domain"
    reports = {"MITRE ATT&CK": ["TA0003:T1098"]}
    default_reference = "https://goteleport.com/docs/management/admin/"
    default_runbook = "A user authenticated with SAML, but from an unknown company domain\n"
    summary_attributes = ["event", "code", "user", "method", "mfa_device"]
    tests = teleport_saml_login_without_company_domain_tests
    TELEPORT_COMPANY_DOMAINS_REGEX = "@(" + "|".join(config.TELEPORT_ORGANIZATION_DOMAINS) + ")$"

    def rule(self, event):
        return (
            event.get("event") == "user.login"
            and event.get("success") is True
            and (event.get("method") == "saml")
            and (not re.search(self.TELEPORT_COMPANY_DOMAINS_REGEX, event.get("user")))
        )

    def title(self, event):
        return f"User [{event.get('user', '<UNKNOWN_USER>')}] logged into [{event.get('cluster_name', '<UNNAMED_CLUSTER>')}] using SAML, but not from a known company domain in ({','.join(config.TELEPORT_ORGANIZATION_DOMAINS)})"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get, get_val_from_list

okta_password_access_tests: list[RuleTest] = [
    RuleTest(
        name="User accessed their own password",
        expected_result=False,
        log={
            "actor": {
                "alternateId": "eric.montgomery@email.com",
                "displayName": "Eric Montgomery",
                "id": "XXXXXXXXXXXXXXXX",
                "type": "User",
            },
            "authenticationContext": {"authenticationStep": 0, "externalSessionId": "XXXXXXXXXXXXXXXXX"},
            "client": {
                "device": "Mobile",
                "geographicalContext": {
                    "country": "Iceland",
                    "geolocation": {"lat": 81.09596, "lon": -10.30578},
                    "state": "Colorado",
                },
                "ipAddress": "218.56.201.220",
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Android 1.x",
                    "rawUserAgent": "Mozilla/5.0 (Linux; Android 11; ONEPLUS A6013) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/104.0.0.0 Mobile Safari/537.36",
                },
                "zone": "null",
            },
            "debugContext": {"debugData": ""},
            "eventType": "application.user_membership.show_password",
            "legacyEventType": "app.generic.show.password",
            "outcome": {"result": "SUCCESS"},
            "p_any_domain_names": ["."],
            "p_any_emails": ["eric.montgomery@email.com"],
            "p_any_ip_addresses": ["218.56.201.220"],
            "p_log_type": "Okta.SystemLog",
            "published": "2022-09-09 04:26:09.792",
            "request": {
                "ipChain": [
                    {
                        "geographicalContext": {
                            "country": "Iceland",
                            "geolocation": {"lat": 81.0959, "lon": -104.9868},
                        },
                        "ip": "218.56.201.220",
                        "version": "V4",
                    },
                ],
            },
            "securityContext": {
                "asNumber": 940252,
                "asOrg": "t-mobile",
                "domain": ".",
                "isProxy": False,
                "isp": "t-mobile usa  inc.",
            },
            "severity": "INFO",
            "target": [
                {
                    "alternateId": "eric.montgomery@email.com",
                    "displayName": "Eric Montgomery",
                    "id": "16442344346b2385",
                    "type": "AppUser",
                },
                {
                    "alternateId": "Application2",
                    "displayName": "Application2",
                    "id": "16442ew83428795",
                    "type": "AppInstance",
                },
                {
                    "alternateId": "eric.montgomery@email.com",
                    "displayName": "Eric Montgomery",
                    "id": "16325kd349753",
                    "type": "User",
                },
            ],
            "transaction": {"detail": {}, "id": "XXXXXXXXXXXXXXXX", "type": "WEB"},
            "uuid": "XXXXXXXXXXXXXXXX",
            "version": "0",
        },
    ),
    RuleTest(
        name="User accessed another user's password",
        expected_result=True,
        log={
            "actor": {
                "alternateId": "eric.montgomery@email.com",
                "displayName": "Eric Montgomery",
                "id": "XXXXXXXXXXXXXXXX",
                "type": "User",
            },
            "authenticationContext": {"authenticationStep": 0, "externalSessionId": "XXXXXXXXXXXXXXXXX"},
            "client": {
                "device": "Mobile",
                "geographicalContext": {
                    "country": "Iceland",
                    "geolocation": {"lat": 81.0959, "lon": -10.30578},
                    "state": "Colorado",
                },
                "ipAddress": "218.56.201.220",
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Android 1.x",
                    "rawUserAgent": "Mozilla/5.0 (Linux; Android 11; ONEPLUS A6013) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/104.0.0.0 Mobile Safari/537.36",
                },
                "zone": "null",
            },
            "debugContext": {"debugData": ""},
            "eventType": "application.user_membership.show_password",
            "legacyEventType": "app.generic.show.password",
            "outcome": {"result": "SUCCESS"},
            "p_any_domain_names": ["."],
            "p_any_emails": ["eric.montgomery@email.com"],
            "p_any_ip_addresses": ["218.56.201.220"],
            "p_log_type": "Okta.SystemLog",
            "published": "2022-09-09 04:26:09.792",
            "request": {
                "ipChain": [
                    {
                        "geographicalContext": {
                            "country": "Iceland",
                            "geolocation": {"lat": 81.09596, "lon": -10.30578},
                        },
                        "ip": "218.56.201.220",
                        "version": "V4",
                    },
                ],
            },
            "securityContext": {
                "asNumber": 124526,
                "asOrg": "t-mobile",
                "domain": ".",
                "isProxy": False,
                "isp": "t-mobile usa  inc.",
            },
            "severity": "INFO",
            "target": [
                {
                    "alternateId": "vanessajohns@email.com",
                    "displayName": "Vanessa Johns",
                    "id": "0uat6tr9otyvdJbBM696",
                    "type": "AppUser",
                },
                {
                    "alternateId": "Application3",
                    "displayName": "Application3",
                    "id": "0oas6wl204Dn3gG5D696",
                    "type": "AppInstance",
                },
                {
                    "alternateId": "vanessajohns@email.com",
                    "displayName": "Vanessa Johns",
                    "id": "XXXXXXXXXXXXXXXX",
                    "type": "User",
                },
            ],
            "transaction": {"detail": {}, "id": "XXXXXXXXXXXXXXXX", "type": "WEB"},
            "uuid": "XXXXXXXXXXXXXXXX",
            "version": "0",
        },
    ),
    RuleTest(
        name="User accessed their own password - 2",
        expected_result=False,
        log={
            "actor": {
                "alternateId": "john.doe@emaildomain.com",
                "displayName": "John Doe",
                "id": "00u3nwfjxxxxxxxxxxxx",
                "type": "User",
            },
            "authenticationContext": {"authenticationStep": 0, "externalSessionId": "XXXXXXXXXXXXXXXXX"},
            "client": {
                "device": "Mobile",
                "geographicalContext": {
                    "country": "Iceland",
                    "geolocation": {"lat": 81.09596, "lon": -10.30578},
                    "state": "Colorado",
                },
                "ipAddress": "218.56.201.220",
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Android 1.x",
                    "rawUserAgent": "Mozilla/5.0 (Linux; Android 11; ONEPLUS A6013) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/104.0.0.0 Mobile Safari/537.36",
                },
                "zone": "null",
            },
            "debugContext": {"debugData": ""},
            "eventType": "application.user_membership.show_password",
            "legacyEventType": "app.generic.show.password",
            "outcome": {"result": "SUCCESS"},
            "published": "2024-03-27 13:17:41.835000000",
            "severity": "INFO",
            "securityContext": {
                "asNumber": 940252,
                "asOrg": "t-mobile",
                "domain": ".",
                "isProxy": False,
                "isp": "t-mobile usa  inc.",
            },
            "target": [
                {"alternateId": "John Doe", "displayName": "John Doe", "id": "00u3nwfjxxxxxxxxxxxx", "type": "AppUser"},
                {
                    "alternateId": "Software",
                    "displayName": "On The Fly App",
                    "id": "11u3nwfjxxxxxxxxxxxx",
                    "type": "AppInstance",
                },
                {
                    "alternateId": "john.doe@emaildomain.com",
                    "displayName": "John Doe",
                    "id": "00u3nwfjxxxxxxxxxxxx",
                    "type": "User",
                },
            ],
            "transaction": {"detail": {}, "id": "XXXXXXXXXXXXXXXX", "type": "WEB"},
            "uuid": "XXXXXXXXXXXXXXXX",
            "version": "0",
        },
    ),
]


@panther_managed
class OktaPasswordAccess(Rule):
    id = "Okta.PasswordAccess-prototype"
    display_name = "Okta Password Accessed"
    log_types = [LogType.OKTA_SYSTEM_LOG]
    tags = ["Okta", "Credential Access:Unsecured Credentials"]
    reports = {"MITRE ATT&CK": ["TA0006:T1552"]}
    default_severity = Severity.MEDIUM
    default_description = "User accessed another user's application password\n"
    default_reference = "https://help.okta.com/en-us/content/topics/apps/apps_revealing_the_password.htm"
    default_runbook = "Investigate whether this was authorized access.\n"
    tests = okta_password_access_tests

    def rule(self, event):
        if event.get("eventType") != "application.user_membership.show_password":
            return False
        # event['target'] = [{...}, {...}, {...}]
        self.TARGET_USERS = get_val_from_list(event.get("target", [{}]), "alternateId", "type", "User")
        self.TARGET_APP_NAMES = get_val_from_list(event.get("target", [{}]), "alternateId", "type", "AppInstance")
        if deep_get(event, "actor", "alternateId") not in self.TARGET_USERS:
            return True
        return False

    def dedup(self, event):
        dedup_str = deep_get(event, "actor", "alternateId")
        if self.TARGET_USERS:
            dedup_str += ":" + str(self.TARGET_USERS)
        if self.TARGET_APP_NAMES:
            dedup_str += ":" + str(self.TARGET_APP_NAMES)
        return dedup_str or ""

    def title(self, event):
        return f"A user {deep_get(event, 'actor', 'alternateId')} accessed another user's {self.TARGET_USERS} {self.TARGET_APP_NAMES} password"
